/*
 * $Id: libgnac-gst.c 276 2009-03-24 15:49:10Z bdupasqu $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <glib/gi18n.h>

#include "libgnac-converter.h"
#include "libgnac-debug.h"
#include "libgnac-error.h"
#include "libgnac-gst.h"


void
libgnac_gst_run(LibgnacConverterItem  *item, 
                GError               **error)
{
  g_return_if_fail(!error || !*error);

  gst_element_set_state(item->pipeline, GST_STATE_PLAYING);
}


void
libgnac_gst_pause(LibgnacConverterItem  *item,
                  GError               **error)
{
  g_return_if_fail(!error || !*error);
  
  gst_element_set_state(item->pipeline, GST_STATE_PAUSED);
}


void
libgnac_gst_stop(LibgnacConverterItem  *item, 
                 GError               **error)
{
  g_return_if_fail(!error || !*error);

  if (item->pipeline) {
    libgnac_gst_clean_pipeline(item);
  }
}


void
libgnac_gst_build_pipeline(LibgnacConverterItem  *item, 
                           GError               **error)
{
  gchar      *description;
  GError     *encoder_error = NULL; 
  GstElement *source = NULL;
  GstElement *decodebin;
  GstElement *sink = NULL;
  GstElement *encoder;

  g_return_if_fail(!error || !*error);

  item->pipeline = gst_pipeline_new("pipeline");
  if (!item->pipeline) {
    libgnac_debug("Pipeline creation failed");

    g_set_error(error,  
     LIBGNAC_ERROR, 
     LIBGNAC_ERROR_PIPELINE_CREATION, 
     _("Unable to create pipeline"));

    return;
  }

  libgnac_output_build_output(item, &encoder_error);
  if (encoder_error) {
    libgnac_debug("Output creation failed");
    g_propagate_error(error, encoder_error);
    return;
  }

  item->bus = gst_element_get_bus(GST_ELEMENT(item->pipeline));
  gst_bus_add_signal_watch(item->bus);

  if (gst_element_factory_find("giosrc")) {
    source = gst_element_factory_make("giosrc", "file-source");
    g_object_set(G_OBJECT(source), "file", item->source, NULL);
  } else {
    libgnac_debug("Failed to find giosrc");

    g_set_error(error,  
     LIBGNAC_ERROR, 
     LIBGNAC_ERROR_PIPELINE_CREATION, 
     _("Unable to find giosrc"));
    return;
  }
  if (!source) {
    libgnac_debug("Source creation failed");

    g_set_error(error,  
     LIBGNAC_ERROR, 
     LIBGNAC_ERROR_PIPELINE_CREATION, 
     _("Unable to create source"));
    return;
  }

  decodebin = gst_element_factory_make("decodebin", "decodebin");
  if (!decodebin) {
    libgnac_debug("Decoder creation failed");
 
    g_set_error(error,  
     LIBGNAC_ERROR, 
     LIBGNAC_ERROR_PIPELINE_CREATION, 
     _("Unable to create decoder"));

    return;
  }

  g_object_get(G_OBJECT(item->converter), "pipeline-description", &description, NULL);

  description = g_strdup_printf("audioresample ! audioconvert ! %s", description);
  encoder = gst_parse_bin_from_description(description, TRUE, &encoder_error);
  g_free(description);
  if (encoder_error) {
    libgnac_debug("Encoder creation failed");
    g_propagate_error(error, encoder_error);
    return;
  }

  item->audio_encoder = encoder;

  if (gst_element_factory_find("giosink")) {
    sink = gst_element_factory_make("giosink", "sink");
    g_object_set(G_OBJECT(sink), "file", item->destination, NULL);
  } else {
    libgnac_debug("Failed to find giosink");

    g_set_error(error,  
     LIBGNAC_ERROR, 
     LIBGNAC_ERROR_PIPELINE_CREATION, 
     _("Unable to find giosink"));
    return;
  }

  if (!sink) {
    libgnac_debug("Sink creation failed");
    g_set_error(error,  
     LIBGNAC_ERROR, 
     LIBGNAC_ERROR_PIPELINE_CREATION, 
     _("Unable to create sink"));

    return;
  }

  /* Connect callbacks */
  
  g_signal_connect(G_OBJECT(item->bus), "message::element", 
      G_CALLBACK(libgnac_gst_element_cb), item);
  /*g_signal_connect(G_OBJECT(bus), "message::tag", 
      G_CALLBACK(libgnac_gst_tags_cb), item);*/
  g_signal_connect(G_OBJECT(item->bus), "message::eos", 
      G_CALLBACK(libgnac_gst_eos_cb), item);
  g_signal_connect(G_OBJECT(item->bus), "message::error", 
      G_CALLBACK(libgnac_gst_error_cb), item);

  g_signal_connect(decodebin, "new-decoded-pad", 
      G_CALLBACK(libgnac_gst_newpad_cb), item);
  g_signal_connect(decodebin, "unknown-type", 
      G_CALLBACK(libgnac_gst_unknown_type_cb), item);

  /* Converter Callbacks */

  g_signal_connect(G_OBJECT(item->bus), "message::error", 
      G_CALLBACK(libgnac_converter_error_cb), item);
  g_signal_connect(G_OBJECT(item->bus), "message::eos", 
      G_CALLBACK(libgnac_converter_eos_cb), item);

  gst_bin_add_many(GST_BIN(item->pipeline), source, decodebin, encoder, sink, NULL);
  gst_element_link(source, decodebin);
  gst_element_link(encoder, sink);

  gst_object_unref(item->bus);
}


void
libgnac_gst_clean_pipeline(LibgnacConverterItem *item)
{
  if (item->pipeline) {
    if (!gst_element_set_state(item->pipeline, GST_STATE_NULL))
    { 
      libgnac_debug("Unable to set state to null");
      return;
    }
    g_object_unref(GST_OBJECT(item->pipeline));
    item->pipeline = NULL;
  }
}


void
libgnac_gst_newpad_cb(GstElement *decodebin,
                      GstPad     *pad,
                      gboolean    last,
                      gpointer    data)
{
	GstCaps *caps;
	GstStructure *structure;
	const gchar *mimetype;
  LibgnacConverterItem *item;

  item = (LibgnacConverterItem*)data;
	caps = gst_pad_get_caps(pad);

	/* we get "ANY" caps for text/plain files etc. */
	if (gst_caps_is_empty(caps) || gst_caps_is_any(caps)) {
    /* NON AUDIO FILE */
	} else {
		/* is this pad audio? */
		structure = gst_caps_get_structure(caps, 0);
		mimetype = gst_structure_get_name(structure);

		if (g_str_has_prefix (mimetype, "audio/x-raw")) {
      // contains audio data
		  GstPad *sink_pad = NULL;
		  sink_pad = gst_element_get_compatible_pad(item->audio_encoder, pad, caps);
      if (GST_PAD_IS_LINKED(sink_pad)) {
        g_object_unref(sink_pad);
        return;
      }

      if (sink_pad) {
		    gst_pad_link(pad, sink_pad);
        g_object_unref(sink_pad);
      } else {
        libgnac_debug("Unable to find a compatible pad");
      }

		} else if (g_str_has_prefix(mimetype, "video/")) {
      /* contains video data */
		} else {
      /* other, we have mimetype */
		}
	}

	gst_caps_unref(caps);
}


void
libgnac_gst_eos_cb(GstBus     *bus, 
                   GstMessage *message, 
                   gpointer    data)
{
  LibgnacConverterItem *item;

  item = (LibgnacConverterItem*)data;
  libgnac_gst_clean_pipeline(item);
}


void
libgnac_gst_error_cb(GstBus     *bus, 
                     GstMessage *message, 
                     gpointer    data)
{
  LibgnacConverterItem *item;

  item = (LibgnacConverterItem*)data;

  libgnac_gst_clean_pipeline(item);
}


/*void
libgnac_gst_tags_cb(GstBus     *bus,
                    GstMessage *message,
                    gpointer    data)
{
		GstTagList *tags;

		gst_message_parse_tag(message, &tags);
		if (tags) {
      // do something with tags
			gst_tag_list_free (tags);
		} else {

		}
}*/


void
libgnac_gst_element_cb(GstBus     *bus,
                       GstMessage *message,
                       gpointer    data)
{
  gchar *details;

  /* This code handle missing plugins installer */
  if (gst_is_missing_plugin_message(message))
  {
    // TODO free details, but where ?
    details = gst_missing_plugin_message_get_installer_detail(message);
    if (details)
    {
      gst_install_plugins_async(&details, NULL, 
            libgnac_converter_missing_plugin_result_cb, data);
    }
  }
}


void
libgnac_gst_unknown_type_cb(GstElement *decodebin, 
                            GstPad     *pad, 
                            GstCaps    *caps, 
                            gpointer    data)
{
  /*
	if (!gst_caps_is_empty (caps) && !gst_caps_is_any (caps)) {
		GstStructure *structure;
		const gchar *mimetype;

		structure = gst_caps_get_structure (caps, 0);
		mimetype = gst_structure_get_name (structure);

		g_free (md->priv->type);
		md->priv->type = g_strdup (mimetype);

		rb_debug ("decodebin emitted unknown type signal for %s", mimetype);
	} else {
		rb_debug ("decodebin emitted unknown type signal");
	}

	md->priv->has_non_audio = TRUE;
#ifndef HAVE_GSTREAMER_0_10_MISSING_PLUGINS
	{
		char *msg;

		msg = make_undecodable_error (md);
		GST_ELEMENT_ERROR (md->priv->pipeline, STREAM, CODEC_NOT_FOUND, ("%s", msg), (NULL));
		g_free (msg);
	}
#endif
  */
}
