/*
 * $Id: libgnac-output.c 383 2009-07-31 14:27:37Z bdupasqu $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <gio/gio.h>
#include <glib.h>
#include <glib/gi18n.h>
#include <gst/gst.h>
#include <stdarg.h>
#include <string.h>

#include "libgnac-debug.h"
#include "libgnac-error.h"
#include "libgnac-metadata.h"
#include "libgnac-output.h"

extern LibgnacMetadata *metadata;

GSList *rename_patterns = NULL;

const gchar rename_pattern_id[] = {
  RENAME_PATTERN_ARTIST,
  RENAME_PATTERN_ALBUM,
  RENAME_PATTERN_COMMENT,
  RENAME_PATTERN_DATE,
  RENAME_PATTERN_DISC_NUMBER,
  RENAME_PATTERN_DISC_COUNT,
  RENAME_PATTERN_FILENAME,
  RENAME_PATTERN_GENRE,
  RENAME_PATTERN_TITLE,
  RENAME_PATTERN_TRACK_COUNT,
  RENAME_PATTERN_TRACK_NUMBER,
  -1,
};

static gchar *
libgnac_output_replace_pattern(LibgnacRenamePattern *pattern,
                               LibgnacTags          *tags);

static LibgnacRenamePattern *
libgnac_output_rename_pattern_new(const gchar pattern);

static void
libgnac_output_rename_patterns_free(void);

static void
libgnac_output_rename_pattern_set_replace(LibgnacRenamePattern *pattern,
                                          LibgnacTags          *tags);


static gchar *
libgnac_output_remove_extension(const gchar *filename)
{
  gchar *ext;
  gchar *output;
  glong  ext_pos;
  glong  output_long;

  g_return_val_if_fail(filename, NULL);

  ext = g_utf8_strrchr(filename, -1, '.');

  ext_pos = ext ? g_utf8_strlen(ext, -1) : 0;
  output_long = g_utf8_strlen(filename, -1) - ext_pos;

  output = g_malloc((output_long + 1) * sizeof(gchar));
  g_utf8_strncpy(output, filename, output_long);

  return output;
}


static gchar *
libgnac_output_remove_extension_from_file(GFile   *source_uri, 
                                          GError **error)
{
  const gchar *filename;
  GError      *err = NULL;
  GFileInfo   *info;

  g_return_val_if_fail(!error || !*error, NULL);
  g_return_val_if_fail(source_uri, NULL);

  info = g_file_query_info(source_uri, 
            G_FILE_ATTRIBUTE_STANDARD_NAME, G_FILE_QUERY_INFO_NONE, 
            NULL, &err);

  if (err) {
    libgnac_debug("Unable to query GFile information");
    g_propagate_error(error, err);
    return NULL;
  }

  filename = g_file_info_get_name(info);
  g_object_unref(info);

  return libgnac_output_remove_extension(filename);
}


static gchar*
libgnac_output_sanitize_path(const gchar *str,
                             gboolean     strip_special)
{
  gchar *res = NULL;
  gchar *s;
  
  /* Skip leading periods, otherwise files disappear... */
  while (*str == '.') str++;
  
  s = g_strdup(str);

  /* XXX we cannot do that because of the custom patterns */
  /* Replace path seperators with a hyphen */
  //g_strdelimit (s, G_DIR_SEPARATOR_S, '-');
  
  // TODO filesystem specific sanitizing
    
  if (strip_special) 
  {
    /* Replace separators with a hyphen */
    g_strdelimit(s, "\\:|", '-');
    /* Replace all other weird characters to whitespace */
    g_strdelimit(s, "*?&!\'\"$`{}()[]<>", ' ');
    /* Replace all whitespace with underscores */
    g_strdelimit(s, "\t ", '_');
  }

  res = g_filename_from_utf8(s, -1, NULL, NULL, NULL);
  g_free(s);
  return res ? res : g_strdup(str);
}


static LibgnacRenamePattern *
libgnac_output_rename_pattern_new(const gchar pattern)
{
  g_return_val_if_fail(pattern, NULL);

  gchar *full_pattern;
  LibgnacRenamePattern *rename_pattern;
  
  rename_pattern = g_malloc(sizeof(LibgnacRenamePattern));
  g_return_val_if_fail(rename_pattern, NULL);

  full_pattern = g_strdup_printf("%c%c", 
                  RENAME_PATTERN_SEPARATOR, pattern);

  rename_pattern->id = pattern;
  rename_pattern->regex = g_regex_new(full_pattern, G_REGEX_OPTIMIZE, 0, NULL);
  g_free(full_pattern);
  
  return rename_pattern;
}


static void
libgnac_output_rename_pattern_set_replace(LibgnacRenamePattern *pattern,
                                          LibgnacTags          *tags)
{
  pattern->replace = libgnac_output_replace_pattern(pattern, tags);
}


static gchar *
libgnac_output_replace_pattern(LibgnacRenamePattern *pattern, 
                               LibgnacTags          *tags)
{
  g_return_val_if_fail(pattern, NULL);

  GValue *val = NULL;

  switch (LIBGNAC_RENAME_PATTERN_GET_ID(pattern))
  {
    case RENAME_PATTERN_ARTIST: 
    {
      if (libgnac_metadata_tag_exists(tags, GST_TAG_ARTIST))
      {
        val = LIBGNAC_METADATA_TAG_ARTIST(tags);
        return g_strdup(g_value_get_string(val));
      }
      return RENAME_PATTERN_DEFAULT_ARTIST;
    }
    case RENAME_PATTERN_ALBUM:
    {
      if (libgnac_metadata_tag_exists(tags, GST_TAG_ALBUM))
      {
        val = LIBGNAC_METADATA_TAG_ALBUM(tags);
        return g_strdup(g_value_get_string(val));
      } 
      return RENAME_PATTERN_DEFAULT_ALBUM;
    }
    case RENAME_PATTERN_DISC_NUMBER:
    {
      if (libgnac_metadata_tag_exists(tags, GST_TAG_ALBUM_VOLUME_NUMBER))
      {
        val = LIBGNAC_METADATA_TAG_ALBUM_VOLUME_NUMBER(tags);
        return g_strdup_printf("%d", g_value_get_uint(val));
      } 
      return RENAME_PATTERN_DEFAULT_DISC_NUMBER;
    }
    case RENAME_PATTERN_DISC_COUNT:
    {
      if (libgnac_metadata_tag_exists(tags, GST_TAG_ALBUM_VOLUME_COUNT))
      {
        val = LIBGNAC_METADATA_TAG_ALBUM_VOLUME_COUNT(tags);
        return g_strdup_printf("%d", g_value_get_uint(val));
      } 
      return RENAME_PATTERN_DEFAULT_DISC_NUMBER;
    }
    case RENAME_PATTERN_COMMENT:
    {
      if (libgnac_metadata_tag_exists(tags, GST_TAG_COMMENT))
      {
        val = LIBGNAC_METADATA_TAG_COMMENT(tags);
        return g_strdup(g_value_get_string(val));
      } 
      return RENAME_PATTERN_DEFAULT_COMMENT;
    }
    case RENAME_PATTERN_DATE:
    {
      if (libgnac_metadata_tag_exists(tags, GST_TAG_DATE))
      {
        val = LIBGNAC_METADATA_TAG_DATE(tags);
        return g_strdup_printf("%d", g_value_get_uint(val));
      } 
      return RENAME_PATTERN_DEFAULT_DATE;
    }
    case RENAME_PATTERN_FILENAME:
    {
      if (libgnac_metadata_tag_exists(tags, GNAC_TAG_FILENAME))
      {
        val = LIBGNAC_METADATA_TAG_FILENAME(tags);
        return g_strdup(libgnac_output_remove_extension(g_value_get_string(val)));
      }
      return RENAME_PATTERN_DEFAULT_FILENAME;
    }
    case RENAME_PATTERN_GENRE:
    {
      if (libgnac_metadata_tag_exists(tags, GST_TAG_GENRE))
      {
        val = LIBGNAC_METADATA_TAG_GENRE(tags);
        return g_strdup(g_value_get_string(val));
      } 
      return RENAME_PATTERN_DEFAULT_GENRE;
    }
    case RENAME_PATTERN_TITLE:
    {
      if (libgnac_metadata_tag_exists(tags, GST_TAG_TITLE))
      {
        val = LIBGNAC_METADATA_TAG_TITLE(tags);
        return g_strdup(g_value_get_string(val));
      } 
      return RENAME_PATTERN_DEFAULT_TITLE;
    }
    case RENAME_PATTERN_TRACK_COUNT:
    {
      if (libgnac_metadata_tag_exists(tags, GST_TAG_TRACK_COUNT))
      {
        val = LIBGNAC_METADATA_TAG_TRACK_COUNT(tags);
        return g_strdup_printf("%02d", g_value_get_uint(val));
      } 
      return RENAME_PATTERN_DEFAULT_TRACK_NUMBER;
    }
    case RENAME_PATTERN_TRACK_NUMBER:
    {
      if (libgnac_metadata_tag_exists(tags, GST_TAG_TRACK_NUMBER))
      {
        val = LIBGNAC_METADATA_TAG_TRACK_NUMBER(tags);
        return g_strdup_printf("%02d", g_value_get_uint(val));
      } 
      return RENAME_PATTERN_DEFAULT_TRACK_NUMBER;
    }
    default:
      libgnac_debug("Unknown pattern: %s", pattern);
  }
  return g_strdup_printf("%s", LIBGNAC_RENAME_PATTERN_GET_PATTERN(pattern));
}


static void
libgnac_output_rename_patterns_init(LibgnacTags *tags)
{
  if (!rename_patterns)
  {
    gint i;
    for (i = 0; rename_pattern_id[i] != -1; i++)
    {
      rename_patterns = g_slist_append(rename_patterns, 
                libgnac_output_rename_pattern_new(rename_pattern_id[i]));
    };
  }
  /* update the replacement values according to 
   * the tags of the current file */
  g_slist_foreach(rename_patterns, 
                  (GFunc)libgnac_output_rename_pattern_set_replace, tags);
}


static void
libgnac_output_rename_patterns_free(void)
{
  g_slist_free(rename_patterns);
}


static void
libgnac_output_rename_pattern_process(LibgnacRenamePattern  *pattern, 
                                      gchar                **output)
{
  *output = g_regex_replace(pattern->regex, *output, -1, 0,
                            pattern->replace, 0, NULL);
}


static gchar *
libgnac_output_get_filename(LibgnacConverterItem *item, 
                            const gchar          *rename_pattern, 
                            GError              **error)
{
  GError      *err = NULL;
  GFile       *file = NULL;
  LibgnacTags *tags = NULL;
  gchar       *output = NULL;

  file = item->source;

  tags = libgnac_metadata_extract(metadata, item->source, NULL);
  if (!tags) 
  {
    output = libgnac_output_remove_extension_from_file(file, &err);
    if (err) 
    {
      libgnac_debug("Unable to remove extension");
      g_propagate_error(error, err);
      return NULL;
    }
  } 
  else 
  {
    libgnac_output_rename_patterns_init(tags);

    output = g_strdup(rename_pattern);

    /* replace all known patterns by their value for the given file */
    g_slist_foreach(rename_patterns, 
                    (GFunc)libgnac_output_rename_pattern_process, &output);
  }

  return output;
}


void
libgnac_output_finalize(void)
{
  libgnac_output_rename_patterns_free();
}


void
libgnac_output_build_output(LibgnacConverterItem  *item,
                            GError               **error)
{
  GError *output_error = NULL;
  GFile  *out_directory;
  GFile  *parent;
  gchar  *extension;
  gchar  *filename = NULL;
  gchar  *folder_hierarchy;
  gchar  *folder_path;
  gchar  *output_name;
  gchar  *sanitized;
  gchar  *rename_pattern;
  gboolean strip_special;
  GnacFolderId folder_type;

  g_return_if_fail(!error || !*error);

  g_object_get(item->converter, "extension", &extension, NULL);
  g_object_get(item->converter, "strip-special", &strip_special, NULL);
  g_object_get(item->converter, "folder-hierarchy", &folder_hierarchy, NULL); 
  g_object_get(item->converter, "folder-path", &folder_path, NULL); 
  g_object_get(item->converter, "rename-pattern", &rename_pattern, NULL);
  g_object_get(item->converter, "folder-type", &folder_type, NULL);

  filename = libgnac_output_get_filename(item, rename_pattern, &output_error);
  if (output_error) {  
    libgnac_debug("Filename creation failed");
    g_propagate_error(error, output_error);
    return;
  }
  
  /*if (!filename) {
    g_set_error(error,
                LIBGNAC_ERROR,
                LIBGNAC_ERROR_INVALID_FILENAME,
                "Invalid output filename");
  }*/

  switch (folder_type)
  {
    case FOLDER_SUBDIRECTORY:
      parent = g_file_get_parent(item->source);
      out_directory = g_file_get_child(parent, folder_path);
      g_object_unref(parent);
    break;

    case FOLDER_SELECTED:
      out_directory = g_file_new_for_uri(folder_path);
    break;

    case FOLDER_CURRENT:
    default:
      out_directory = g_file_get_parent(item->source);
    break;
  }

  /* check whether we have to build a folder hierarchy */
  if (!g_str_equal(g_strstrip(folder_hierarchy), "")) 
  {
    /* replace all known patterns by their value */
    g_slist_foreach(rename_patterns,
        (GFunc)libgnac_output_rename_pattern_process, &folder_hierarchy);
    /* build the output filename */
    output_name = g_strdup_printf("%s%c%s.%s", 
                    folder_hierarchy, G_DIR_SEPARATOR, filename, extension);
  }
  else
  {
    output_name = g_strdup_printf("%s.%s", filename, extension);
  }

  /* sanitize the whole path */
  sanitized = libgnac_output_sanitize_path(output_name, strip_special);

  /* create the folder hierarchy and destination file */
  item->destination = g_file_get_child(out_directory, sanitized);

  /* cleanup */
  g_free(extension);
  g_free(folder_hierarchy);
  g_free(folder_path);
  g_free(rename_pattern);
  g_free(output_name);
  g_free(sanitized);
  g_free(filename);
}


gchar *
libgnac_output_get_preview_from_pattern(const gchar *pattern,
                                        gboolean     strip_special)
{
  gchar *preview;
  gchar *sanitized;
  LibgnacTags *tags;

  preview = g_strdup(pattern);
  tags = libgnac_metadata_get_dummy_tags();
  libgnac_output_rename_patterns_init(tags);

  /* replace all known patterns by their value for the given file */
  g_slist_foreach(rename_patterns, 
      (GFunc)libgnac_output_rename_pattern_process, &preview);

  /* sanitize the filename */
  sanitized = libgnac_output_sanitize_path(preview, strip_special);

  /* cleanup */
  g_free(preview);
  
  return sanitized;
}
