/*
 * $Id: gnac-properties.c 366 2009-07-08 20:57:30Z bdupasqu $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <glib/gi18n.h>
#include <gst/gst.h>

#include "gnac-file-list.h"
#include "gnac-properties.h"
#include "gnac-ui.h"
#include "gnac-utils.h"

#include "libgnac-metadata.h"


static void 
gnac_properties_set_entry(const gchar  *entry_name, 
                          const GValue *entry_value);

static void
gnac_properties_build_table(void);

static gchar *
gnac_properties_get_property_from_value(const gchar  *name, 
                                        const GValue *value);
static void
gnac_properties_set_property(const gchar  *name, 
                             const GValue *value);

static void
gnac_properties_reset_spin_if_empty(const gchar *name);


static GtkBuilder   *gnac_properties_builder = NULL;
static GHashTable   *gnac_properties_table;
static GtkTreeModel *properties_filter;

static GtkWidget *gnac_properties_window;
static GtkWidget *backward_arrow;
static GtkWidget *forward_arrow;

static gboolean properties_displayed = FALSE;

static GtkTreeRowReference *current_ref;

static const gchar *displayed_tags[] = {
  GST_TAG_ALBUM,
  GST_TAG_ALBUM_VOLUME_COUNT,
  GST_TAG_ALBUM_VOLUME_NUMBER,
  GST_TAG_ARTIST,
  GST_TAG_COMMENT,
  GST_TAG_DATE,
  GST_TAG_GENRE,
  GST_TAG_IMAGE,
  GST_TAG_TITLE,
  GST_TAG_TRACK_COUNT,
  GST_TAG_TRACK_NUMBER,
  NULL
};

static const gchar *displayed_properties[] = {
  GNAC_TAG_FILENAME,
  GST_TAG_LOCATION,
  GST_TAG_DURATION,
  GNAC_TAG_FILE_SIZE,
  GNAC_TAG_CHANNELS,
  GNAC_TAG_MODE,
  GNAC_TAG_RATE,
  GST_TAG_AUDIO_CODEC,
  GST_TAG_BITRATE,
  GST_TAG_TRACK_GAIN,
  GST_TAG_TRACK_PEAK,
  NULL
};

static const gchar *nice_properties[] = {
  N_("Filename"),
  N_("Location"),
  N_("Duration"),
  N_("File size"),
  N_("Channels"),
  N_("Mode"),
  N_("Rate"),
  N_("Codec"),
  N_("Bitrate"),
  N_("Track Gain"),
  N_("Track Peak"),
  NULL
};


extern LibgnacMetadata *metadata;


static gboolean
gnac_properties_visible_func(GtkTreeModel *model,
                             GtkTreeIter  *iter,
                             gpointer      data)
{
  /* Visible if PROPERTY_VISIBLE is true */
  gboolean visible;

  gtk_tree_model_get (model, iter, PROPERTY_VISIBLE, &visible, -1);

  return visible;
}


static void
gnac_properties_build_table(void)
{
  GtkCellRenderer *renderer;
  GtkTreeModel *model;
  GtkWidget *view;

  GtkTreeRowReference *ref = NULL;
  GtkTreeIter iter;
  gint i;

  model = GTK_TREE_MODEL(gtk_builder_get_object(gnac_properties_builder, 
                                            "properties_store"));
  view = GTK_WIDGET(gtk_builder_get_object(gnac_properties_builder, 
                                            "properties_view"));

  gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(view), PROPERTY_TOOLTIP);

  properties_filter = gtk_tree_model_filter_new(model, NULL);
  gtk_tree_model_filter_set_visible_func(
        GTK_TREE_MODEL_FILTER(properties_filter), 
        gnac_properties_visible_func, NULL, NULL);
  gtk_tree_view_set_model(GTK_TREE_VIEW(view), properties_filter);

  renderer = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(renderer), "xalign", 1.0, NULL);
  g_object_set(G_OBJECT(renderer), "xpad", 6, NULL);
  g_object_set(G_OBJECT(renderer), "weight", 700, NULL);
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
                                              -1,
                                              "Property",
                                              renderer,
                                              "text",
                                              PROPERTY_NAME,
                                              NULL);

  renderer = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(renderer), "ellipsize", PANGO_ELLIPSIZE_END, NULL);
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
                                              -1,
                                              "Value",
                                              renderer,
                                              "text",
                                              PROPERTY_VALUE,
                                              NULL);

  gnac_properties_table = g_hash_table_new_full(g_str_hash, 
                                                g_str_equal, 
                                                g_free, 
                (GDestroyNotify)gtk_tree_row_reference_free);
  
  for (i = 0; displayed_properties[i]; i++) 
  {
    GtkTreePath *path;

    gtk_list_store_append(GTK_LIST_STORE(model), &iter);
    gtk_list_store_set(GTK_LIST_STORE(model), &iter,
                    PROPERTY_NAME, gettext(nice_properties[i]),
                    PROPERTY_VISIBLE, TRUE, -1);

    path = gtk_tree_model_get_path(model, &iter);
    ref = gtk_tree_row_reference_new(model, path);

    g_hash_table_insert(gnac_properties_table, 
                    g_strdup(displayed_properties[i]), ref);

    gtk_tree_path_free(path);
  }
}


static void
gnac_properties_reset_properties(void)
{
  gint i;

  /* reset the basic tab */
  for (i = 0; displayed_tags[i]; i++) {
    gnac_properties_set_entry(displayed_tags[i], NULL);
  }

  for (i = 0; displayed_properties[i]; i++) {
    gnac_properties_set_property(displayed_properties[i], NULL);
  }

  gtk_tree_model_filter_refilter(GTK_TREE_MODEL_FILTER(properties_filter));
}


static void
gnac_properties_set_property(const gchar  *name, 
                             const GValue *value)
{
  gchar *str_value = NULL;
  GtkTreeRowReference *ref = NULL;
  GtkTreeModel *model;
  GtkTreePath *path;
  gboolean visible = TRUE;

  if (value) {
    str_value = gnac_properties_get_property_from_value(name, value);
    if (!str_value) visible = FALSE;
  } else visible = FALSE;

  model = GTK_TREE_MODEL(gtk_builder_get_object(gnac_properties_builder, 
                                            "properties_store"));
  ref = g_hash_table_lookup(gnac_properties_table, name);
  if (ref) {
    gchar *desc = NULL;
    gchar *tooltip = NULL;

    path = gtk_tree_row_reference_get_path(ref);
    if (path) {

      GtkTreeIter iter;

      if (gtk_tree_model_get_iter(model, &iter, path)) 
      {
        gtk_tree_model_get(model, &iter, 
                        PROPERTY_VALUE, &desc,
                        PROPERTY_TOOLTIP, &tooltip,
                        -1);

        gtk_list_store_set(GTK_LIST_STORE(model), &iter,
                        PROPERTY_VALUE, str_value,
                        PROPERTY_VISIBLE, visible,
                        -1);

        if (LIBGNAC_METADATA_TAG_IS_LOCATION(name) || 
            LIBGNAC_METADATA_TAG_IS_FILENAME(name)) 
        {
          gtk_list_store_set(GTK_LIST_STORE(model), &iter,
                          PROPERTY_TOOLTIP, str_value,
                          -1);
        }

        g_free(desc);
        g_free(tooltip);
      }
    }
    g_free(str_value);
    gtk_tree_path_free(path);
    gtk_tree_model_filter_refilter(GTK_TREE_MODEL_FILTER(properties_filter));
  }
}


static void 
gnac_properties_set_entry(const gchar  *entry_name, 
                          const GValue *entry_value)
{
  GtkWidget *entry = NULL;
  gdouble value;

  entry = GTK_WIDGET(gtk_builder_get_object(gnac_properties_builder, entry_name));
  if (!entry) 
  {
    gnac_properties_set_property(entry_name, entry_value);
    return;
  }

  /* if there is no value for this field, reset it */
  if (!entry_value) 
  {
    if (GTK_IS_ENTRY(entry)) 
    {
      gtk_entry_set_text(GTK_ENTRY(entry), "");
    } 
    else if (GTK_IS_IMAGE(entry)) 
    {
      /* TODO look for a cover in the current directory?
       *      (folder.jpg, cover.jpg, front.jpg, ...) */
      gtk_image_clear(GTK_IMAGE(entry));
    }
    return;
  }

  if (LIBGNAC_METADATA_TAG_IS_IMAGE(entry_name)) 
  {
    GdkPixbuf *pixbuf = g_value_get_object(entry_value);
    pixbuf = gnac_utils_scale_pixbuf(pixbuf, 96, 96);
    pixbuf = gnac_utils_add_border_to_pixbuf(pixbuf);
    gtk_image_set_from_pixbuf(GTK_IMAGE(entry), pixbuf);
    return;
  }

  GType type = G_VALUE_TYPE(entry_value);
  switch (type) {

    case G_TYPE_STRING:
      gtk_entry_set_text(GTK_ENTRY(entry), 
                      g_value_get_string(entry_value));
      return;

    case G_TYPE_UINT:
      value = (gdouble) g_value_get_uint(entry_value);
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(entry), value);
      gtk_spin_button_set_range(GTK_SPIN_BUTTON(entry), value, value);
      return;
    
    default:
      gtk_entry_set_text(GTK_ENTRY(entry), 
                      g_strdup_value_contents(entry_value));
      g_printerr(_("%s had an unknown type: %s\n"), 
                      entry_name, g_type_name(type));
      return;
  }
}


static gchar *
gnac_properties_get_property_from_value(const gchar  *name,
                                        const GValue *value)
{
  if (LIBGNAC_METADATA_TAG_IS_DURATION(name))
  {
    guint64 duration;
    duration = g_value_get_uint64(value);
    return gnac_utils_format_duration_for_display(duration);
  }
  if (LIBGNAC_METADATA_TAG_IS_BITRATE(name))
  {
    guint val = g_value_get_uint(value);
    if (val == 0) return NULL; 
    else return g_strdup_printf("%d kbps", val);
  }
  if (LIBGNAC_METADATA_TAG_IS_FILE_SIZE(name)) 
  {
    gint64 val = g_value_get_int64(value);
    if (val == 0) return NULL;
    gchar *size;
    size = g_format_size_for_display(val);
    return size;
  }
  if (LIBGNAC_METADATA_TAG_IS_RATE(name)) 
  {
    gint val = g_value_get_int(value);
    if (val == 0) return NULL;
    else return g_strdup_printf("%d Hz", val);
  }
  if (LIBGNAC_METADATA_TAG_IS_CHANNELS(name)) 
  {
    gint val = g_value_get_int(value);
    if (val == 0) return NULL;
    else return g_strdup_printf("%d", val);
  }
  if (LIBGNAC_METADATA_TAG_IS_FILENAME(name) || 
           LIBGNAC_METADATA_TAG_IS_LOCATION(name) ||
           LIBGNAC_METADATA_TAG_IS_AUDIO_CODEC(name) ||
           LIBGNAC_METADATA_TAG_IS_TRACK_GAIN(name) ||
           LIBGNAC_METADATA_TAG_IS_TRACK_PEAK(name) ||
           LIBGNAC_METADATA_TAG_IS_MODE(name))
  {
    return g_value_dup_string(value);
  }
  return NULL;
}


GtkWidget *
gnac_properties_window_new(void)
{
  GError *err = NULL;
  GtkWidget *properties_window;

  gnac_properties_builder = gtk_builder_new();

  gtk_builder_add_from_file(gnac_properties_builder, 
                            PKGDATADIR "/gnac-properties-window.xml", &err);
  if (err != NULL) 
  {
    g_printerr(_("Unable to read file: %s\n"), err->message);
    g_error_free (err);
  } 
  g_assert (err == NULL && gnac_properties_builder != NULL);

  gtk_builder_connect_signals(gnac_properties_builder, NULL);

  /* Build properties table */
  gnac_properties_build_table();

  properties_window = GTK_WIDGET(gtk_builder_get_object(gnac_properties_builder, 
                                            "properties_window"));
  backward_arrow = GTK_WIDGET(gtk_builder_get_object(gnac_properties_builder, 
                                                    "backward_btn"));
  forward_arrow = GTK_WIDGET(gtk_builder_get_object(gnac_properties_builder, 
                                                          "forward_btn"));
  gtk_window_set_transient_for(GTK_WINDOW(properties_window), 
                                GTK_WINDOW(gnac_ui_get_widget("main_window")));
  gtk_window_set_type_hint(GTK_WINDOW(properties_window), GDK_WINDOW_TYPE_HINT_DIALOG);

  return properties_window;
}


void
gnac_properties_update_display(GFile *file)
{
  if (properties_displayed) 
  {
    LibgnacTags *tags;
    tags = libgnac_metadata_extract(metadata, file, NULL); 
    gnac_properties_reset_properties();
    if (tags) 
    {
      g_hash_table_foreach(tags, (GHFunc)gnac_properties_set_entry, NULL);
    }
  }
}


void
gnac_properties_update_arrows(void)
{
  if (backward_arrow && forward_arrow && current_ref) 
  {
    /* forward arrow */
    if (gnac_file_list_has_next_row(current_ref)) {
      gtk_widget_set_sensitive(forward_arrow, TRUE);
    } else {
      gtk_widget_set_sensitive(forward_arrow, FALSE);
    }
    /* backward arrow */
    if (gnac_file_list_has_prev_row(current_ref)) {
      gtk_widget_set_sensitive(backward_arrow, TRUE);
    } else {
      gtk_widget_set_sensitive(backward_arrow, FALSE);
    }
  }
}


static void
gnac_properties_reset_spin_if_empty(const gchar *name)
{
  GtkWidget *spin = NULL;

  spin = GTK_WIDGET(gtk_builder_get_object(gnac_properties_builder, 
                                            name));

  if (gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin)) == 0) {
    gtk_entry_set_text(GTK_ENTRY(spin), "");
  }
}


void
gnac_properties_window_show(void)
{
  gchar *file_uri;
  GtkWidget *notebook;

  if (!gnac_properties_window) {
    gnac_properties_window = gnac_properties_window_new();
  }

  notebook = GTK_WIDGET(gtk_builder_get_object(gnac_properties_builder, 
                                            "notebook1"));

  properties_displayed = TRUE;

  if (gnac_file_list_get_current_row(&current_ref)) 
  {
    gnac_properties_update_arrows();
    if (gnac_file_list_get(current_ref, &file_uri)) 
    {
      GFile *uri;

      uri = g_file_new_for_uri(file_uri);
      gnac_properties_update_display(uri);
      g_object_unref(uri);

      g_free(file_uri);
    }
  }

  gtk_widget_show_all(gnac_properties_window);

  gnac_properties_reset_spin_if_empty("track-number");
  gnac_properties_reset_spin_if_empty("track-count");
  gnac_properties_reset_spin_if_empty("album-disc-number");
  gnac_properties_reset_spin_if_empty("album-disc-count");
  gnac_properties_reset_spin_if_empty("date");

  gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 0);
  gtk_window_present(GTK_WINDOW(gnac_properties_window));
}


void
gnac_properties_window_hide(void)
{
  if (gnac_properties_window) {
    gtk_widget_hide_all(gnac_properties_window);
    properties_displayed = FALSE;
  }
}


void
gnac_properties_set_row(GtkTreeRowReference *current)
{
  gchar *file_uri;

  gtk_tree_row_reference_free(current_ref);
  current_ref = NULL;

  if (gtk_tree_row_reference_valid(current)) 
  {
    current_ref = gtk_tree_row_reference_copy(current);
    gnac_properties_update_arrows();
    gnac_file_list_select_row_and_follow(current_ref);
    if (gnac_file_list_get(current_ref, &file_uri)) 
    {
      GFile *uri;

      uri = g_file_new_for_uri(file_uri);
      gnac_properties_update_display(uri);
      g_object_unref(uri);

      g_free(file_uri);
    }
  }
}


void
gnac_on_back(GtkWidget *widget,
             gpointer   data)
{
  GtkTreeRowReference *old_ref;

  gchar *file_uri;

  old_ref = gtk_tree_row_reference_copy(current_ref);

  if (gnac_file_list_get_prev_row(&current_ref)) 
  {
    gnac_file_list_select_row_and_follow(current_ref);

    if (gnac_file_list_get(current_ref, &file_uri)) 
    {
      GFile *uri;

      uri = g_file_new_for_uri(file_uri);
      gnac_properties_update_display(uri);
      g_object_unref(uri);

      g_free(file_uri);
    }

    gtk_tree_row_reference_free(old_ref);

  } else {
    gtk_tree_row_reference_free(current_ref);
    current_ref = old_ref;
  }

  gnac_properties_update_arrows();
}


void
gnac_on_forward(GtkWidget *widget, 
                gpointer   data)
{
  GtkTreeRowReference *old_ref;
  
  gchar *file_uri;

  old_ref = gtk_tree_row_reference_copy(current_ref);

  if (gnac_file_list_get_next_row(&current_ref)) 
  {
    gnac_file_list_select_row_and_follow(current_ref);

    if (gnac_file_list_get(current_ref, &file_uri)) 
    {
      GFile *uri;

      uri = g_file_new_for_uri(file_uri);
      gnac_properties_update_display(uri);
      g_object_unref(uri);

      g_free(file_uri);
    }

    gtk_tree_row_reference_free(old_ref);
  
  } else {
    gtk_tree_row_reference_free(current_ref);
    current_ref = old_ref;   
  }

  gnac_properties_update_arrows();
}


void
gnac_on_close_properties(GtkWidget *widget,
                         gpointer   data)
{
  gnac_properties_window_hide();
}


void
gnac_properties_destroy(void)
{
  GtkWidget *window = NULL;
  if (gnac_properties_builder) {
    window = GTK_WIDGET(gtk_builder_get_object(gnac_properties_builder, 
                                            "properties_window"));
    g_object_unref(gnac_properties_builder);
    gnac_properties_builder = NULL;
  }

  if (window) 
    gtk_widget_destroy(window);
}
