/*
 * $Id: gnac-profiles-manager.c 379 2009-07-22 17:35:09Z bdupasqu $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <gio/gio.h>
#include <gdk/gdkkeysyms.h>
#include <glib/gi18n.h>

#include "gnac-profiles-manager.h"
#include "libgnac-debug.h"

#define GNAC_COPY_SUFFIX N_(" (copy)")

enum {
  COL_NAME,
  COL_FORMAT,
  COL_EXTENSION,
  COL_PROFILE,
  NUM_COLS
};

enum {
  TARGET_URI,
  TARGET_PLAIN,
} DRAG_AND_DROP_TARGETS;

GtkBuilder *profiles_mgr_builder = NULL;

static GtkTargetEntry target_list[] =  {
  { "text/uri-list", 0, TARGET_URI },
  { "text/plain", 0, TARGET_PLAIN }
};

static guint n_targets = G_N_ELEMENTS(target_list);
static StandardCallBack close_callback;
static gboolean first_show = TRUE;

static void
gnac_profiles_mgr_populate(void);

static void
gnac_profiles_mgr_import_default_profiles(void);

static void
gnac_profiles_mgr_clear(void);

static void
gnac_profiles_mgr_insert(void *profile);

static void
gnac_profiles_mgr_remove(GtkTreeRowReference *ref);

static void
gnac_profiles_mgr_delete_profile_file(const gchar *name);

static void
gnac_profiles_mgr_get(GtkTreeRowReference *ref, int col_number, void **data);

static void
gnac_profiles_mgr_set(GtkTreeRowReference *ref, ...);

static GList *
gnac_profiles_mgr_get_selected_rows(void);

static void
gnac_profiles_mgr_activate_buttons(gboolean activate);

static void
gnac_profiles_mgr_show_description_frame(gboolean show);

static void
gnac_profiles_mgr_show_import_progressbar(gboolean show);

static void
gnac_profiles_mgr_set_progress_bar_fraction(gdouble fraction);

static void
gnac_profiles_mgr_display_status_message(const gchar *ok, const gchar *error);

static void
gnac_profiles_mgr_set_window_sensitive(gboolean sensitive);

static void
gnac_profiles_mgr_copy_and_load_files(gpointer data);

static gchar **
gnac_profiles_mgr_get_selected_uris(void);

static void
gnac_profiles_mgr_on_drag_profile_copied( goffset current_num_bytes,
                                      goffset total_num_bytes,
                                      gpointer user_data);

static void
gnac_profiles_mgr_on_edit_profile(GtkWidget *widget, gpointer data);

static void
gnac_profiles_mgr_on_new_profile(GtkWidget *widget, gpointer data);

static void
gnac_profiles_mgr_on_treeselection_changed(void);

void
gnac_profiles_mgr_init(StandardCallBack call_back)
{
  GtkTreeSelection *selection;
  GtkCellRenderer *renderer;
  GtkTreeView *view;
  GtkTreeModel *model;
  GtkWidget *widget;
  GError *error = NULL;

  close_callback = call_back;

  profiles_mgr_builder = gtk_builder_new();
  gtk_builder_add_from_file(profiles_mgr_builder, 
    PKGDATADIR "/profiles/gnac-profiles-manager.xml", &error);
  if (error) {
    libgnac_critical("%s\n", error->message);
    g_error_free(error);
    exit(EXIT_FAILURE);
  }

  view = GTK_TREE_VIEW(gtk_builder_get_object(profiles_mgr_builder,
    "profile_treeview"));
  model = gtk_tree_view_get_model(view);

  selection = gtk_tree_view_get_selection(view);
  g_signal_connect(G_OBJECT(selection), "changed", 
      (GCallback) gnac_profiles_mgr_on_treeselection_changed, NULL);

  renderer = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(renderer), "ypad", 5, NULL);
  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
                                              -1,
                                              _("Name"),
                                              renderer,
                                              "text",
                                              COL_NAME,
                                              NULL);

  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
                                              -1,
                                              _("Format"),
                                              renderer,
                                              "text",
                                              COL_FORMAT,
                                              NULL);

  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
                                              -1,
                                              _("Extension"),
                                              renderer,
                                              "text",
                                              COL_EXTENSION,
                                              NULL);
  
  gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model), 0, GTK_SORT_ASCENDING);

  gtk_drag_dest_set(GTK_WIDGET(view),
      GTK_DEST_DEFAULT_ALL,
      target_list,
      n_targets,
      GDK_ACTION_COPY);


  gtk_drag_source_set(GTK_WIDGET(view),
      GDK_BUTTON1_MASK,
      target_list,
      n_targets,
      GDK_ACTION_COPY);  

  

  widget = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
    "profile_manager_window"));
  gtk_window_set_modal(GTK_WINDOW(widget),TRUE);
  gtk_window_set_type_hint(GTK_WINDOW(widget), GDK_WINDOW_TYPE_HINT_DIALOG);
  
  gnac_profiles_properties_init();

  gnac_profiles_mgr_show_import_progressbar(FALSE);
  gnac_profiles_mgr_display_status_message(NULL,NULL);

  gnac_profiles_mgr_populate();

  gtk_builder_connect_signals(profiles_mgr_builder, NULL);
  
}

void
gnac_profiles_mgr_set_parent(GtkWidget *parent)
{
  GtkWidget *widget;

  widget = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
    "profile_manager_window"));
  gtk_window_set_transient_for(GTK_WINDOW(widget), GTK_WINDOW(parent));
  gtk_window_set_position(GTK_WINDOW(widget), GTK_WIN_POS_CENTER_ON_PARENT);

  gnac_profiles_properties_set_parent(widget);
}

static void
gnac_profiles_mgr_populate()
{ 
  void *profile;
  GFile *dir;
  GFile *profile_file;
  gchar *profile_file_path;
  gchar *profile_file_full_path;
  const gchar *profile_file_name;
  GFileInfo *file_info;
  GFileEnumerator *files;
  GError *error = NULL;
  
  gnac_profiles_mgr_clear();

  dir = g_file_new_for_path(saved_profiles_dir);
  if (!g_file_query_exists(dir,NULL)) {
    if (!g_file_make_directory_with_parents(dir,NULL,&error)) {
      libgnac_warning("%s %s: %s\n",_("Unable to create directory for saved profiles."),
        _("You may not be able to save your profiles"),
        error->message);
      gnac_profiles_mgr_display_status_message(NULL, _("Unable to create directory"
        " for saved profiles."));
      g_error_free(error);
      return;
    } else {
      gnac_profiles_mgr_import_default_profiles();
    }
  }

  /* TODO if the profiles directory exists, but does not 
   * contain any profile, import the default ones */

  files = g_file_enumerate_children(dir,
      G_FILE_ATTRIBUTE_STANDARD_NAME ","
      G_FILE_ATTRIBUTE_STANDARD_TYPE,
      G_FILE_QUERY_INFO_NONE ,NULL,&error);

  if (files != NULL) {
    while ((file_info = g_file_enumerator_next_file(files, NULL, NULL)) != NULL) {
      if (g_file_info_get_file_type(file_info) == G_FILE_TYPE_REGULAR) {
        profile_file = g_file_enumerator_get_container(files);
        profile_file_path = g_file_get_path(profile_file);
        profile_file_name = g_file_info_get_name(file_info);
        profile_file_full_path = g_build_filename(profile_file_path,
          profile_file_name, NULL);
        profile = gnac_profiles_properties_load_profile_from_file(
          profile_file_full_path, profile_file_name, &error);
        if (profile != NULL) {
          gnac_profiles_mgr_insert(profile); 
        } else if (error) {
          libgnac_warning("%s\n",error->message);
          g_clear_error(&error);
        }
        g_free(profile_file_path);
        g_free(profile_file_full_path);
      }
      g_object_unref(G_OBJECT(file_info));
    }
   gnac_profiles_mgr_display_status_message(NULL,NULL);

  } else {
    libgnac_warning("%s: %s\n",_("Unable to browse the profiles directory"),
      error->message);
    gnac_profiles_mgr_display_status_message(NULL, _("Unable to browse the profiles"
      " directory"));
    g_error_free(error);
    return;
  }

  g_object_unref(dir);
  g_file_enumerator_close(files, NULL, NULL);
  g_object_unref(files);
}

static void
gnac_profiles_mgr_import_default_profiles(void) {
  void *profile;
  GFile *dir;
  GFile *profile_file;
  gchar *profile_file_path;
  gchar *profile_file_full_path;
  const gchar *profile_file_name;
  GFileInfo *file_info;
  GFileEnumerator *files;
  GError *error = NULL;
  
  gnac_profiles_mgr_clear();

  dir = g_file_new_for_path(PKGDATADIR "/profiles/default");
  if (!g_file_query_exists(dir,NULL)) {
    libgnac_warning("%s: %s\n",_("Unable to find directory containing def"
        "ault profiles"), error->message);
    g_error_free(error);
    return;
  }
  
  files = g_file_enumerate_children(dir,
      G_FILE_ATTRIBUTE_STANDARD_NAME ","
      G_FILE_ATTRIBUTE_STANDARD_TYPE,
      G_FILE_QUERY_INFO_NONE ,NULL,&error);

  if (files != NULL) {
    while ((file_info = g_file_enumerator_next_file(files, NULL, NULL)) != NULL) {
      if (g_file_info_get_file_type(file_info) == G_FILE_TYPE_REGULAR) {
        profile_file = g_file_enumerator_get_container(files);
        profile_file_path = g_file_get_path(profile_file);
        profile_file_name = g_file_info_get_name(file_info);
        profile_file_full_path = g_build_filename(profile_file_path,
          profile_file_name, NULL);
        profile = gnac_profiles_properties_load_profile_from_file(
          profile_file_full_path, profile_file_name, &error);
        if (profile != NULL) {
          gnac_profiles_properties_save_profile(profile);
        } else if (error) {
          libgnac_warning("%s\n",error->message);
          g_clear_error(&error);
        }
        g_free(profile_file_path);
        g_free(profile_file_full_path);
        gnac_profiles_properties_free_audio_profile(profile);
      }
      g_object_unref(G_OBJECT(file_info));
    }
  } else {
    libgnac_warning("%s: %s\n",_("Unable to browse the default profiles direc"
      "tory"), error->message);
    g_error_free(error);
    return;
  }
  g_object_unref(dir);
  g_file_enumerator_close(files, NULL, NULL);
  g_object_unref(files);
}

GList *
gnac_profiles_mgr_get_profiles_list()
{
  GList *list = NULL;
  GtkTreeModel *model;
  GtkTreeIter iter;
  AudioProfileGeneric *profile;
  
  model = GTK_TREE_MODEL(gtk_builder_get_object(profiles_mgr_builder,
    "liststore"));
  if (gtk_tree_model_get_iter_first(model,&iter)) {
    do {
      gtk_tree_model_get(model,&iter,COL_PROFILE,(void**)&profile,-1);
      list = g_list_append(list, (gpointer) profile);
    } while (gtk_tree_model_iter_next(model, &iter));
  }
  return list;
}

void
gnac_profiles_mgr_destroy()
{
  GtkWidget *widget;
  
  gnac_profiles_mgr_clear();

  if (profiles_mgr_builder) {
    widget = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
      "profile_manager_window"));
    g_object_unref(profiles_mgr_builder);
    gtk_widget_destroy(widget);
  }

  gnac_profiles_properties_destroy();
}

void
gnac_profiles_mgr_show()
{
  GtkWidget *window;
  GList *selected;
 
  if (!first_show) {
    gnac_profiles_mgr_populate();
  } else {
    first_show = FALSE;
  }

  window = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
    "profile_manager_window"));
  gtk_widget_show(window);

  selected = gnac_profiles_mgr_get_selected_rows();
  if (selected == NULL) {
    gnac_profiles_mgr_show_description_frame(FALSE);
    gnac_profiles_mgr_activate_buttons(FALSE);
  }

  g_list_foreach (selected, (GFunc)gtk_tree_row_reference_free, NULL);
  g_list_free (selected);
  g_mem_profile ();
}

static void
gnac_profiles_mgr_clear(void)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  AudioProfileGeneric *profile;
  
  model = GTK_TREE_MODEL(gtk_builder_get_object(profiles_mgr_builder,
    "liststore"));
  if (gtk_tree_model_get_iter_first(model,&iter)) {
    do {
      gtk_tree_model_get(model,&iter,COL_PROFILE,(void**)&profile,-1);
      gnac_profiles_properties_free_audio_profile(profile);
    } while (gtk_tree_model_iter_next(model, &iter));
  }
  gtk_list_store_clear(GTK_LIST_STORE(model));
}

static void
gnac_profiles_mgr_insert(void *profile)
{
  GtkListStore *model;
  GtkTreeIter iter;
  AudioProfileGeneric *generic;
  gchar *formated_name;

  generic = ((AudioProfileGeneric*) profile)->generic;
  model = GTK_LIST_STORE(gtk_builder_get_object(profiles_mgr_builder,
    "liststore"));

  formated_name = gnac_profiles_properties_filter_text_for_displaying(
    generic->name, MAX_NAME_DISPLAY_SIZE);
  gtk_list_store_append(model, &iter);
  gtk_list_store_set(model, &iter,
      0, formated_name,
      1, generic->format_name,
      2, generic->extension,
      3, profile,
      -1);
  g_free(formated_name);

}

static void
gnac_profiles_mgr_remove(GtkTreeRowReference *ref)
{
  GtkTreeModel *model;
  GtkTreePath *path;
  void *profile = NULL;

  g_return_if_fail(ref);

  model = GTK_TREE_MODEL(gtk_builder_get_object(profiles_mgr_builder, "liststore"));
  path = gtk_tree_row_reference_get_path(ref);

  if (path) {
    GtkTreeIter iter;

    if (gtk_tree_model_get_iter(model, &iter, path)) {
      gtk_tree_model_get(model, &iter,
          COL_PROFILE, &profile,
          -1);
      gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
    }
  }

  gtk_tree_path_free(path);

  if (profile) {
    gnac_profiles_properties_free_audio_profile((AudioProfileGeneric*)profile);
  }
  
}

static void
gnac_profiles_mgr_get(GtkTreeRowReference *ref, int col_number, void **data)
{
  GtkTreeModel *model;
  GtkTreePath *path;

  g_return_if_fail(ref);

  model = GTK_TREE_MODEL(gtk_builder_get_object(profiles_mgr_builder, "liststore"));
  path = gtk_tree_row_reference_get_path(ref);

  if (path) {
    GtkTreeIter iter;

    if (gtk_tree_model_get_iter(model, &iter, path)) {
      gtk_tree_model_get(model, &iter, col_number, data, -1);
    }
  }

  gtk_tree_path_free(path);

}

static void
gnac_profiles_mgr_set(GtkTreeRowReference *ref, ...)
{
  GtkListStore *list_store;
  GtkTreePath *path;
  va_list ap;

  g_return_if_fail(ref);

  list_store = GTK_LIST_STORE(gtk_builder_get_object(profiles_mgr_builder, "liststore"));
  path = gtk_tree_row_reference_get_path(ref);

  if (path) {
    GtkTreeIter iter;
    
    if (gtk_tree_model_get_iter(GTK_TREE_MODEL(list_store), &iter, path)) {
      va_start(ap,ref);
      gtk_list_store_set_valist(list_store, &iter, ap);
    }
  }

  gtk_tree_path_free(path);
}


static GList *
gnac_profiles_mgr_get_selected_rows(void)
{
  GtkTreeView *view;
  GtkTreeModel *model;
  GtkTreeSelection *selection;

  GList *row_references = NULL ;
  GList *next;
  GList *next_temp;
  
  model = GTK_TREE_MODEL(gtk_builder_get_object(profiles_mgr_builder, "liststore"));
  view = GTK_TREE_VIEW(gtk_builder_get_object(profiles_mgr_builder, "profile_treeview"));
  selection = gtk_tree_view_get_selection(view);

  /* Convert path to GtkTreeRowReference */
  next = gtk_tree_selection_get_selected_rows(selection, &model);
  next_temp = next;
  while (next) {
    row_references = g_list_prepend(row_references, 
        gtk_tree_row_reference_new(model, next->data));
    next = next->next;
  }

  row_references = g_list_reverse(row_references);
  g_list_foreach (next_temp, (GFunc) gtk_tree_path_free, NULL);
  g_list_free (next_temp);

  return row_references;
}

static void
gnac_profiles_mgr_activate_buttons(gboolean activate)
{
  GtkWidget *widget;

  widget = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder, "copy_button"));
  gtk_widget_set_sensitive(widget, activate);

  widget = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder, "edit_button"));
  gtk_widget_set_sensitive(widget, activate);

  widget = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder, "delete_button"));
  gtk_widget_set_sensitive(widget, activate);
}

static void
gnac_profiles_mgr_show_description_frame(gboolean show)
{
  GtkWidget *widget;
  
  widget = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
    "frame-description"));
  if (show) {
    gtk_widget_show_all(widget);
  } else {
    gtk_widget_hide_all(widget);
  }
    
}

static void
gnac_profiles_mgr_show_import_progressbar(gboolean show)
{
  GtkWidget *widget;
  widget = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
    "frame-import"));

  if (show) {
    gtk_widget_show_all(widget);
  } else {
    gtk_widget_hide_all(widget);
  }
}

static void
gnac_profiles_mgr_set_window_sensitive(gboolean sensitive)
{
  GtkWidget *widget;
  widget = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
    "frame-profiles-list"));
  gtk_widget_set_sensitive(widget, sensitive);

  widget = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
    "frame-description"));
  gtk_widget_set_sensitive(widget, sensitive);

  widget = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
    "frame-status"));
  gtk_widget_set_sensitive(widget, sensitive);

  widget = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
    "close_button"));
  gtk_widget_set_sensitive(widget, sensitive);

}

static void
gnac_profiles_mgr_set_progress_bar_fraction(gdouble fraction)
{
  GtkWidget *widget;
    
  widget = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
    "progressbar-import"));
  gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(widget), fraction);
}

static void
gnac_profiles_mgr_display_status_message(const gchar *ok, const gchar *error)
{
  GtkWidget *frame_status;
  GtkWidget *label_status_ok;
  GtkWidget *image_status_ok;
  GtkWidget *label_status_error;
  GtkWidget *image_status_error;
  
  frame_status = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
    "frame-status"));

  if ((ok == NULL) && (error == NULL)) {
    gtk_widget_hide_all(frame_status);
  } else {
    gtk_widget_show_all(frame_status);
    label_status_ok = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
      "label-status-ok"));
    image_status_ok = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
      "image-status-ok"));
    label_status_error = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
      "label-status-error"));
    image_status_error = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
      "image-status-error"));
    if (ok != NULL) {
      gtk_label_set_markup(GTK_LABEL(label_status_ok),ok);
      gtk_widget_show(label_status_ok);
      gtk_widget_show(image_status_ok);
    } else {
      gtk_widget_hide(label_status_ok);
      gtk_widget_hide(image_status_ok);
    }
    if (error != NULL) {
      gtk_label_set_markup(GTK_LABEL(label_status_error),error);
      gtk_widget_show(label_status_error);
      gtk_widget_show(image_status_error);
    } else {
      gtk_widget_hide(label_status_error);
      gtk_widget_hide(image_status_error);
    }
  }

}

typedef struct {
  gchar **uris;
  guint info;
} ThreadCopyData;

void
gnac_profiles_mgr_on_drag_data_received( GtkWidget* widget, 
  GdkDragContext *context, gint x, gint y, GtkSelectionData *selection_data,
  guint info, guint time, gpointer data)
{
  gchar **uris;
  GError *error = NULL;
  ThreadCopyData *tcopy_data;
  
  tcopy_data = g_malloc(sizeof(ThreadCopyData));
  uris = g_uri_list_extract_uris((const gchar*) selection_data->data);

  if (uris == NULL) {
    gtk_drag_finish (context, FALSE, FALSE, time);
    return;
  }
  tcopy_data->uris = uris;
  tcopy_data->info = info;
  g_thread_create((GThreadFunc) gnac_profiles_mgr_copy_and_load_files, tcopy_data, 
    TRUE, &error);
  if (error) {
    libgnac_warning(_("Impossible to launch thread for collecting files to import: %s\n"),
      error->message);
    gnac_profiles_mgr_display_status_message(NULL,_("Impossible to import file(s)"));
    g_error_free(error); 
  }
  gtk_drag_finish(context, TRUE, FALSE, time);

}

typedef struct {
  const gchar *path;
  const gchar *name;
  GError **error;
  gboolean toggle; 
} CopyData;

static void
gnac_profiles_mgr_copy_and_load_files(gpointer data)
{
  gchar **uris;
  gchar *uri;
  GError *error = NULL;
  guint profiles_ok = 0;
  guint profiles_error = 0;
  gint index = 0;
  const gchar *name;
  ThreadCopyData *tcopy_data;

  tcopy_data = (ThreadCopyData*) data;
  uris = tcopy_data->uris;

  uri = uris[index];

  gdk_threads_enter();
  gnac_profiles_mgr_set_window_sensitive(FALSE);
  gnac_profiles_mgr_show_import_progressbar(TRUE);
  gdk_flush();
  gdk_threads_leave();
  
  while (uri != NULL) {
    GFile *file;
    GFileInfo *file_info;

    file = g_file_new_for_uri(uri);
    file_info = g_file_query_info(file,
      G_FILE_ATTRIBUTE_STANDARD_NAME "," G_FILE_ATTRIBUTE_STANDARD_TYPE,
      G_FILE_QUERY_INFO_NONE ,NULL, &error);
    if (error) {
      libgnac_warning("%s\n",error->message);
      ++profiles_error;
      g_clear_error(&error);
    } else {
      gboolean ok = FALSE;

      switch (tcopy_data->info) {
        case TARGET_URI:
          if (g_file_info_get_file_type(file_info) == G_FILE_TYPE_REGULAR) {
            name = g_file_info_get_name(file_info);
            if (name != NULL) ok = TRUE;
          }
          break;
        case TARGET_PLAIN:
          if (g_file_info_get_file_type(file_info) == G_FILE_TYPE_UNKNOWN) {
            name = g_file_get_basename(file);
            if (name != NULL) ok = TRUE;
          }
          break;
        default:
          return;
      }

      if (ok) {
        if (gnac_profiles_properties_saved_profiles_contain_name(name,FALSE)) {
          libgnac_warning(_("Impossible to load file \"%s\": a profile with same file "
            "name already exists.\n"), uri);
          ++profiles_error;
        } else {
          gchar *path;
          GFile *dest;
          CopyData copy_data;

          path = g_build_filename(g_get_tmp_dir(), name, NULL);
          dest = g_file_new_for_path(path);
          copy_data.path = path;
          copy_data.name = name;
          copy_data.error = &error;
          copy_data.toggle = FALSE;

          gdk_threads_enter();
          gnac_profiles_mgr_set_progress_bar_fraction(0.0);
          gdk_threads_leave(); 
          
          g_file_copy(file, dest, G_FILE_COPY_NONE, NULL, 
            (GFileProgressCallback) gnac_profiles_mgr_on_drag_profile_copied, 
            &copy_data, &error);
          
          if (error) {
            libgnac_warning("%s\n",error->message);
            g_clear_error(&error);
            ++profiles_error;
          } else {
            ++profiles_ok;
          }
          g_free(path);
          g_object_unref(dest);
        }
      } else {
        libgnac_warning(_("Impossible to import the file \"%s\". File type not supported\n"),
          uri);
        ++profiles_error;
      }
    }
    ++index;
    uri = uris[index];
    g_object_unref(file);
    g_object_unref(file_info);
  }

  gchar *oks = NULL;
  gchar *errors = NULL;
  
  if (profiles_error > 0) {
    errors = g_strdup_printf("%d %s", profiles_error, ((profiles_error > 1) 
      ? _("files failed to be imported") : _("file failed to be imported")));
    if (profiles_ok > 0) {
      oks = g_strdup_printf("%d %s", profiles_ok, ((profiles_ok > 1)
      ? _("files successfully imported") : _("file successfully imported")));
    }
  }

  gdk_threads_enter();
  gnac_profiles_mgr_set_window_sensitive(TRUE);
  gnac_profiles_mgr_show_import_progressbar(FALSE);
  gnac_profiles_mgr_display_status_message(oks,errors);
  gdk_flush();
  gdk_threads_leave();

  g_free(oks);
  g_free(errors);
  g_strfreev(tcopy_data->uris);
  g_free(tcopy_data);
}

void
gnac_profiles_mgr_on_drag_profile_copied( goffset current_num_bytes,
                                      goffset total_num_bytes,
                                      gpointer user_data)
{
  AudioProfileGeneric *profile;
  CopyData *copy_data;
  GFile *file;

  
  gdouble frac =  ((gdouble) current_num_bytes)/ ((gdouble) total_num_bytes);
  if (frac > 1.0) frac = 1.0;
  else if (frac < 0.0) frac = 0.0;
  
  gdk_threads_enter();
  gnac_profiles_mgr_set_progress_bar_fraction(frac);
  gdk_flush();
  gdk_threads_leave();

  if (current_num_bytes >= total_num_bytes) {
    copy_data = (CopyData *) user_data;
    if (!copy_data->toggle) {
      copy_data->toggle = TRUE;
    } else {
      profile = gnac_profiles_properties_load_profile_from_file(copy_data->path,
      copy_data->name, copy_data->error);
      if (profile != NULL) {
        gdk_threads_enter();
        gnac_profiles_properties_save_profile(profile);
        gnac_profiles_mgr_insert(profile);
        gdk_flush();
        gdk_threads_leave();
      }
      file = g_file_new_for_path(copy_data->path);
      g_file_delete(file, NULL, NULL);
      g_object_unref(file);
    } 
  }
}

void
gnac_profiles_mgr_on_drag_data_get(GtkWidget *widget,
  GdkDragContext *drag_context, GtkSelectionData *data, guint info, guint time,
  gpointer user_data)
{
  gchar **uris;

  uris = gnac_profiles_mgr_get_selected_uris();

  gtk_selection_data_set_uris(data, uris);
}

static gchar **
gnac_profiles_mgr_get_selected_uris(void)
{
  GList *selected;
  GList *temp;
  gchar **uris;
  gint i;
  
  selected = gnac_profiles_mgr_get_selected_rows();
  uris = g_malloc((g_list_length(selected)+1) * sizeof(gchar*));
  for (i = 0, temp = selected; temp != NULL; temp = temp->next, ++i) {
    GFile *file;
    AudioProfileGeneric *profile;
    gchar *path;

    gnac_profiles_mgr_get((GtkTreeRowReference*)selected->data, COL_PROFILE,
      (void**) &profile);
    profile = profile->generic;
    path = g_strconcat(GNAC_SAVED_PROFILES_URL_WITH_EXT(profile->name), NULL);
    file = g_file_new_for_path(path);
    uris[i] = g_file_get_uri(file); 
    g_free(path);
  }
  uris[i] = NULL;

  g_list_foreach (selected, (GFunc)gtk_tree_row_reference_free, NULL);
  g_list_free (selected);
  return uris;
}


gboolean
gnac_profiles_mgr_on_drag_drop(GtkWidget *widget, GdkDragContext *drag_context,
  gint x, gint y, guint time, gpointer user_data)
{
  return TRUE;
}


void
gnac_profiles_mgr_on_add(GtkWidget *widget, gpointer data)
{
  gnac_profiles_properties_show(NULL, _("New profile"), 
    G_CALLBACK(gnac_profiles_mgr_on_new_profile)); 
}


static void
gnac_profiles_mgr_on_new_profile(GtkWidget *widget, gpointer data)
{
  gnac_profiles_mgr_insert(data); 
  gnac_profiles_mgr_display_status_message(NULL,NULL);
}

void
gnac_profiles_mgr_on_copy(GtkWidget *widget, 
                          gpointer   data)
{
  AudioProfileGeneric *profile;
  AudioProfileGeneric *generic;
  AudioProfileGeneric *copy_profile;
  gchar  *full_path;
  gchar  *new_path;
  gchar  *new_name;
  GError *error = NULL;
  GFile  *dst;
  GFile  *src;
  GList  *selected;

  selected = gnac_profiles_mgr_get_selected_rows();
  gnac_profiles_mgr_get((GtkTreeRowReference*)selected->data, 
                  COL_PROFILE, (void**)&profile);
  generic = profile->generic;

  full_path = g_strconcat(GNAC_SAVED_PROFILES_URL_WITH_EXT(generic->name), NULL);
  new_name = g_strconcat(GNAC_SAVED_PROFILES_URL(generic->name), 
                         gettext(GNAC_COPY_SUFFIX), NULL);
  new_path = g_strconcat(new_name, ".xml", NULL);

  src = g_file_new_for_path(full_path);
  dst = g_file_new_for_path(new_path);
  
  g_file_copy(src, dst, G_FILE_COPY_NONE, NULL, NULL, NULL, &error);
  if (error)
  {
    libgnac_debug("Failed to copy the profile %s: %s", 
                    generic->name, error->message);
    gnac_profiles_mgr_display_status_message(
                    NULL, _("Failed to copy the profile"));
    g_clear_error(&error);
    g_object_unref(src);
    g_object_unref(dst);
    g_free(full_path);
    g_free(new_path);
    g_free(new_name);
    g_list_foreach(selected, (GFunc)gtk_tree_row_reference_free, NULL);
    g_list_free(selected);
    return;
  }

  copy_profile = gnac_profiles_properties_load_profile_from_file(
                                              new_path, new_name, &error);
  if (copy_profile)
  {
    (copy_profile->generic)->name = g_strconcat(generic->name, 
                                   gettext(GNAC_COPY_SUFFIX), NULL);
    gnac_profiles_properties_save_profile(copy_profile);
    gnac_profiles_mgr_insert(copy_profile); 
  } 
  else if (error) 
  {
    libgnac_debug("Failed to load the profile %s: %s", 
                    (copy_profile->generic)->name, error->message);
    gnac_profiles_mgr_display_status_message(
                    NULL, _("Failed to copy the profile"));
    g_clear_error(&error);
  }

  /* cleanup */
  g_object_unref(src);
  g_object_unref(dst);
  g_free(full_path);
  g_free(new_path);
  g_free(new_name);
  g_list_foreach(selected, (GFunc)gtk_tree_row_reference_free, NULL);
  g_list_free(selected);
}

void
gnac_profiles_mgr_on_edit(GtkWidget *widget, gpointer data)
{
  GList *selected;
  AudioProfileGeneric *profile;

  selected = gnac_profiles_mgr_get_selected_rows();

  gnac_profiles_mgr_get((GtkTreeRowReference*)selected->data, COL_PROFILE, (void**)&profile);
  profile->generic->user_data = selected->data;

  g_list_foreach (selected, (GFunc)gtk_tree_row_reference_free, NULL);
  g_list_free (selected);
  
  gnac_profiles_properties_show(profile,_("Edit profile"), G_CALLBACK(gnac_profiles_mgr_on_edit_profile));
}

static void
gnac_profiles_mgr_on_edit_profile(GtkWidget *widget, gpointer data)
{
  AudioProfileGeneric *profile;
  AudioProfileGeneric *generic;
  AudioProfileGeneric *old_generic;
  void *old_profile;
  gchar *formated_name;
  GList *selected;
  //gchar *message;

  selected = gnac_profiles_mgr_get_selected_rows();
  gnac_profiles_mgr_get((GtkTreeRowReference*)selected->data, COL_PROFILE,
    &old_profile);
  old_generic = ((AudioProfileGeneric*)old_profile)->generic;

  profile = (AudioProfileGeneric*) data;
  generic = profile->generic;

  if (g_strcmp0(generic->name,old_generic->name)) {
    gnac_profiles_mgr_delete_profile_file(old_generic->name);
  }

  formated_name = gnac_profiles_properties_filter_text_for_displaying(
    generic->name, MAX_NAME_DISPLAY_SIZE);
  gnac_profiles_mgr_set((GtkTreeRowReference*) selected->data, 
    0, formated_name, 
    1, generic->format_name,
    2, generic->extension, 
    3, profile,
    -1); 
  
  g_free(formated_name);
  gnac_profiles_properties_free_audio_profile(old_profile);

  g_list_foreach (selected, (GFunc)gtk_tree_row_reference_free, NULL);
  g_list_free (selected);

  gnac_profiles_mgr_display_status_message(NULL,NULL);

  gnac_profiles_mgr_on_treeselection_changed();
}

void
gnac_profiles_mgr_on_remove(GtkWidget *widget, gpointer data)
{
  GList *selected;
  GtkTreeModel *model;
  GtkTreePath *path;
  GtkTreeIter iter;
  gboolean new_selected = FALSE;
  AudioProfileGeneric *profile;

  selected = gnac_profiles_mgr_get_selected_rows();
  if (selected != NULL) { 
    gnac_profiles_mgr_get((GtkTreeRowReference*)selected->data, COL_PROFILE,
      (void**) &profile);
    gnac_profiles_mgr_delete_profile_file(profile->generic->name);
    
    gnac_profiles_mgr_display_status_message(NULL,NULL);

    /* We can do this while selection is unique */
    model = gtk_tree_row_reference_get_model((GtkTreeRowReference*)selected->data);
    path = gtk_tree_row_reference_get_path((GtkTreeRowReference*)selected->data);
    if  (gtk_tree_model_get_iter(model, &iter, path)) {
      if (gtk_tree_model_iter_next(model,&iter)
          || (gtk_tree_path_prev(path) && gtk_tree_model_get_iter(model, &iter, path))) {

        GtkTreeView *view;
        GtkTreeSelection *selection;

        view = GTK_TREE_VIEW(gtk_builder_get_object(profiles_mgr_builder, 
          "profile_treeview"));
        selection = gtk_tree_view_get_selection(view);
        gtk_tree_selection_select_iter(selection, &iter);
        new_selected = TRUE;
      }
    }

    gnac_profiles_mgr_remove((GtkTreeRowReference*)selected->data);
    
    gnac_profiles_mgr_activate_buttons(new_selected);
    gnac_profiles_mgr_show_description_frame(new_selected);

    gtk_tree_path_free(path);
    g_list_foreach (selected, (GFunc)gtk_tree_row_reference_free, NULL);
    g_list_free (selected);
  }

}

static void
gnac_profiles_mgr_delete_profile_file(const gchar *name)
{
  GFile *file;
  gchar *full_path;
  
  full_path = g_strconcat(GNAC_SAVED_PROFILES_URL_WITH_EXT(name) ,NULL);
  file = g_file_new_for_path(full_path);
  g_file_delete(file, NULL, NULL);
  g_object_unref(file);
  g_free(full_path);
}

void
gnac_profiles_mgr_on_row_inserted(GtkTreeModel *tree_model, 
                                  GtkTreePath  *path, 
                                  GtkTreeIter  *iter, 
                                  gpointer      user_data)
{
}

void
gnac_profiles_mgr_on_row_deleted(GtkTreeModel *tree_model, 
                                 GtkTreePath  *path, 
                                 gpointer      user_data)
{
}

static void
gnac_profiles_mgr_on_treeselection_changed(void)
{
  AudioProfileGeneric *profile;
  GList *selected;
  GtkWidget *widget;

  selected = gnac_profiles_mgr_get_selected_rows();
  //When remove
  if (selected != NULL) {
    gchar *descr;

    gnac_profiles_mgr_activate_buttons(TRUE);
    gnac_profiles_mgr_display_status_message(NULL,NULL);
    
    gnac_profiles_mgr_get((GtkTreeRowReference*)selected->data, COL_PROFILE,
      (void**) &profile);
    widget = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder, 
      "label-description-content"));
    descr = gnac_profiles_properties_filter_text_for_displaying(
      profile->generic->description, MAX_DESCR_DISPLAY_SIZE);
    gtk_label_set_text(GTK_LABEL(widget), descr);
    g_free(descr);

    gnac_profiles_mgr_show_description_frame(TRUE);
    
    g_list_foreach (selected, (GFunc)gtk_tree_row_reference_free, NULL);
    g_list_free (selected);
  }
}

void
gnac_profiles_mgr_on_close(GtkWidget *widget, gpointer data)
{
  GtkWidget *window;
  
  window = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
    "profile_manager_window"));
  gtk_widget_hide(window);
  close_callback(widget, data);
}

gboolean
gnac_profiles_mgr_on_delete_event(GtkWidget *widget, GdkEvent *event,
  gpointer user_data)
{
  gnac_profiles_mgr_on_close(widget,user_data);
  return TRUE;
}

gboolean
gnac_profiles_mgr_list_on_key_pressed(GtkWidget *widget,
  GdkEventKey *event, gpointer data)
{
  switch (event->keyval) {
    case GDK_Delete:
      gnac_profiles_mgr_on_remove(NULL,NULL);
      break;
    default:
      break;
  }
  return FALSE;
}

gboolean
gnac_profiles_mgr_on_key_pressed(GtkWidget *widget,
  GdkEventKey *event, gpointer data)
{
  switch (event->keyval) {
    case GDK_Escape:
      widget = GTK_WIDGET(gtk_builder_get_object(profiles_mgr_builder,
        "close_button"));
      gtk_button_clicked(GTK_BUTTON(widget));
      return TRUE;
    default:
      break;
  }
  return FALSE;
}
