/*
 * $Id: gnac-profiles-properties.c 282 2009-03-24 23:34:35Z alexandreroux $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <gdk/gdkkeysyms.h>
#include <gio/gio.h>
#include <gmodule.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n.h>

#include "gnac-profiles-properties.h"
#include "gnac-profiles-default.h"
#include "libgnac-debug.h"


#ifdef _WIN32
  #ifndef __WIN32
    #define __WIN32
  #endif
#endif

#ifdef __WIN32
static const guint nb_forbidden_chars = 11;
//TODO put correct GDK key event
static const char forbidden_chars[] = {'\\', '/', ':', '*', '?', '"', 
                                '<', '>', '|', '{', '}'};
static const gchar *forbidden_chars_string = "\\ / : * ? \" < > | { }";
#else
static const guint nb_forbidden_chars = 3;
static const guint forbidden_chars_keys[] = {GDK_slash}; 
static const gchar forbidden_chars[] = {'/','\n','\t'}; 
static const gchar *forbidden_chars_string = "   /, <return>, <tab>";
#endif

#define NB_PLUGINS 8

#include "formats/gnac-profiles-aac.h"
#include "formats/gnac-profiles-flac.h"
#include "formats/gnac-profiles-lame.h"
#include "formats/gnac-profiles-speex.h"
#include "formats/gnac-profiles-unknown.h"
#include "formats/gnac-profiles-vorbis.h"
#include "formats/gnac-profiles-wav.h"
#include "formats/gnac-profiles-wavpack.h"

static FormatModuleGetFuncs formats_get_funcs[NB_PLUGINS] = {
  gnac_profiles_aac_get_funcs,
  gnac_profiles_flac_get_funcs,
  gnac_profiles_lame_get_funcs,
  gnac_profiles_speex_get_funcs,
  gnac_profiles_unknown_get_funcs,
  gnac_profiles_vorbis_get_funcs,
  gnac_profiles_wav_get_funcs,
  gnac_profiles_wavpack_get_funcs
};

//static WindowSize properties_window_size; 

typedef struct {
  FormatModuleFuncs funcs;
  //GModule *module;
  GtkTreeRowReference *tree_ref;
} FormatModule;

static GHashTable *formats;

static FormatModule *current_format_module;
static GtkBuilder *profiles_properties_builder = NULL;
static gulong save_button_handler_id = 0;
static const gchar *current_profile_name;

static void
gnac_profiles_properties_init_format(void);

/*
static void
gnac_profiles_properties_init_modules(void);

static GList *
gnac_profiles_properties_find_modules(void);*/

static void
gnac_profiles_properties_combo_format_on_changed(GtkComboBox *widget,
  gpointer user_data);

static void
gnac_profiles_properties_update_textbuffer(const gchar *pipeline);

static void
gnac_profiles_properties_display_status_message(const gchar *error);

static gboolean
gnac_profiles_properties_is_valid_filename_chars(const gchar *chars);

static gboolean
gnac_profiles_properties_is_valid_filename_char(guint keyval);

static void
gnac_profiles_properties_on_save(GtkWidget *widget, gpointer data);

static void
gnac_profiles_properties_name_description_set_text(const gchar *name,
  const gchar *description);

static void
gnac_profiles_properties_reinit(void);

void
gnac_profiles_properties_init(void)
{
  GtkWidget *widget;
  GtkWidget *combo_format;
  GtkWidget *hbox_properties;
  GtkCellRenderer *renderer_format;
  
  profiles_properties_builder = gtk_builder_new(); 
  formats = g_hash_table_new(g_str_hash,g_str_equal);
  
  saved_profiles_dir = g_build_filename(g_get_home_dir(), ".gnac", "profiles", 
    NULL);

  GError *error = NULL;
  if (!gtk_builder_add_from_file(profiles_properties_builder, 
    PKGDATADIR "/profiles/gnac-profiles-properties.xml", &error)) {
    libgnac_critical("%s\n",error->message);
    g_error_free(error);
    exit(EXIT_FAILURE);
  }
  
  hbox_properties = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
    "hbox_properties"));

  //Set up combo format
  combo_format = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
    "format_combo_box"));
  gtk_builder_connect_signals (profiles_properties_builder, NULL);         

  renderer_format = gtk_cell_renderer_text_new();
  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo_format), 
    renderer_format, FALSE);
  gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combo_format), 
    renderer_format, "text", 0, NULL);

  g_signal_connect (G_OBJECT (combo_format), "changed",
    G_CALLBACK (gnac_profiles_properties_combo_format_on_changed), hbox_properties);
 
  widget = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,"window1"));
  gtk_window_set_type_hint(GTK_WINDOW(widget), GDK_WINDOW_TYPE_HINT_DIALOG);
  
  g_signal_connect (G_OBJECT (widget), "delete_event",
                    G_CALLBACK (gtk_widget_hide), NULL);

  gtk_builder_connect_signals (profiles_properties_builder, NULL); 

  gnac_profiles_properties_init_format();

  gnac_profiles_properties_display_status_message(NULL);
  
  gtk_window_set_modal(GTK_WINDOW(widget),TRUE);

}

void
gnac_profiles_properties_set_parent(GtkWidget *parent)
{
  GtkWidget *widget;

  widget = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,"window1"));
  
  gtk_window_set_transient_for(GTK_WINDOW(widget), GTK_WINDOW(parent));
  gtk_window_set_position(GTK_WINDOW(widget), GTK_WIN_POS_CENTER_ON_PARENT);
}

static void
gnac_profiles_properties_init_format(void) {
  GtkWidget *widget;
  GtkWidget *format_combo_box;
  GtkWidget *hbox_properties;
  GtkWidget *properties;
  GtkListStore *model;
  GtkTreeIter iter;
  GtkTreePath *tree_path;
  const gchar *format_id;
  gchar *format_name;
  int i;
  
  hbox_properties = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
  "hbox_properties"));
  
  format_combo_box = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
    "format_combo_box"));
  model = GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(format_combo_box)));

  gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model), 2, GTK_SORT_ASCENDING);
 

  for (i = 0; i<NB_PLUGINS; ++i) {
    FormatModule *format_module;
    GtkWidget *pipeline_box;
    
    format_module = g_malloc(sizeof(FormatModule));
    format_module->funcs = formats_get_funcs[i]();
    format_id = format_module->funcs.init(gnac_profiles_properties_update_textbuffer);
    format_name = format_module->funcs.get_combo_format_name();
    g_hash_table_insert(formats, (gpointer) format_id, format_module);
    
    gtk_list_store_append(model, &iter);
    if (g_strcmp0(format_id,"custom-format")) {
      gtk_list_store_set(model, &iter, 0, format_name, 1, format_module,
        2, format_name, -1);
    } else {
      //This is a big hack to be sure that Custom Format will be at the end :-D
      gtk_list_store_set(model, &iter, 0, format_name, 1, format_module,
        2, "ZZZZZZZZZZZZZZZZZZZ", -1);
    }
    tree_path = gtk_tree_model_get_path(GTK_TREE_MODEL(model), &iter);
    format_module->tree_ref = gtk_tree_row_reference_new(
      GTK_TREE_MODEL(model), tree_path);
    gtk_tree_path_free(tree_path);
    
    if (format_module->funcs.set_pipeline_func != NULL) {
      widget = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
        "gstreamer_pipeline_text_view"));
      pipeline_box = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
        "gstreamer_pipeline_box"));
      format_module->funcs.set_pipeline_func(GTK_TEXT_VIEW(widget), pipeline_box);
    }

    properties = format_module->funcs.get_widget(NULL);
    gtk_box_pack_start_defaults(GTK_BOX(hbox_properties), properties);
    gtk_widget_hide(properties);

    g_free(format_name);
  }
}

/*
static void
gnac_profiles_properties_init_modules(void)
{
  GList *list;
  GList *temp;
  GModule *module;
  GtkWidget *widget;
  GtkWidget *format_combo_box;
  GtkWidget *hbox_properties;
  GtkListStore *model;
  GtkTreeIter iter;
  GtkTreePath *tree_path;

  hbox_properties = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
  "hbox_properties"));
  
  list = gnac_profiles_properties_find_modules();
  temp = list;
  format_combo_box = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
    "format_combo_box"));
  model = GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(format_combo_box)));

  gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model), 2, GTK_SORT_ASCENDING);
  
  while (temp != NULL) {
    module = g_module_open(temp->data, G_MODULE_BIND_LAZY); 
    if (module == NULL) {
      libgnac_warning(_("Impossible to load format module %s\n"), (gchar*) temp->data);
    } else {
      FormatModule *format_module;
      FormatModuleFuncs *format_funcs;
      FormatModuleInitFunc module_init;
      FormatModuleGetComboFormatNameFunc get_combo_format_name;
      FormatModuleSetPipelineTextViewFunc set_pipeline_text_view;
      FormatModuleGetPluginName get_plugin_name;
      const gchar *format_id;
      const gchar *plugin_name;
      gchar *format_name;
      GstPlugin *plugin;
      GtkWidget *properties;
      
      g_module_symbol(module, "gnac_profiles_format_init", 
        (gpointer*) &module_init);
      if (module_init) {
        format_id = module_init(gnac_profiles_properties_update_textbuffer);
        
        format_module = g_malloc(sizeof(FormatModule));
        format_module->module = module;
        format_funcs = &(format_module->funcs);
        
        g_module_symbol(module, "gnac_profiles_format_get_description",
          (gpointer*) &format_funcs->get_description);
        g_module_symbol(module, "gnac_profiles_format_generate_pipeline",
          (gpointer*) &format_funcs->generate_pipeline);
        g_module_symbol(module, "gnac_profiles_format_generate_audio_profile",
          (gpointer*) &format_funcs->generate_audio_profile);
        g_module_symbol(module, "gnac_profiles_format_free_audio_profile",
          (gpointer*) &format_funcs->free_audio_profile);
        g_module_symbol(module, "gnac_profiles_format_fullfill_fields",
          (gpointer*) &format_funcs->fullfill_fields);
        g_module_symbol(module, "gnac_profiles_format_get_widget",
          (gpointer*) &format_funcs->get_widget);
        g_module_symbol(module, "gnac_profiles_format_save_profile",
          (gpointer*) &format_funcs->save_profile_in_file);
        g_module_symbol(module, "gnac_profiles_format_load_specific_properties",
          (gpointer*) &format_funcs->load_specific_properties);
        g_module_symbol(module, "gnac_profiles_format_clean_up",
          (gpointer*) &format_funcs->clean_up);
        
        g_module_symbol(module, "gnac_profiles_format_get_plugin_name",
          (gpointer*) &get_plugin_name);
        plugin_name = get_plugin_name();
        plugin = gst_plugin_load_by_name(plugin_name);
        if (plugin != NULL) {
          g_hash_table_insert(formats, (gpointer) format_id, format_module);
          
          g_module_symbol(module, "gnac_profiles_format_get_combo_format_name",
            (gpointer*) &get_combo_format_name);
          format_name = get_combo_format_name();

          gtk_list_store_append(model, &iter);
          if (g_strcmp0(format_id,"custom-format")) {
            gtk_list_store_set(model, &iter, 0, format_name, 1, format_module,
              2, format_name, -1);
          } else {
            //This is a big hack to be sure that Custom Format will be at the end :-D
            gtk_list_store_set(model, &iter, 0, format_name, 1, format_module,
              2, "ZZZZZZZZZZZZZZZZZZZ", -1);
          }
          tree_path = gtk_tree_model_get_path(GTK_TREE_MODEL(model), &iter);
          format_module->tree_ref = gtk_tree_row_reference_new(
            GTK_TREE_MODEL(model), tree_path);
          gtk_tree_path_free(tree_path);
          if (g_module_symbol(module, "gnac_profiles_format_set_pipeline_text_view",
              (gpointer*) &set_pipeline_text_view)) {
            GtkWidget *pipeline_box;
            
            widget = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
              "gstreamer_pipeline_text_view"));
            pipeline_box = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
              "gstreamer_pipeline_box"));
            set_pipeline_text_view(GTK_TEXT_VIEW(widget), pipeline_box);
          }
          properties = format_funcs->get_widget(NULL);
          gtk_box_pack_start_defaults(GTK_BOX(hbox_properties), properties);
          gtk_widget_hide(properties);
          g_free(format_name);
          g_object_unref(G_OBJECT(plugin));
        } else {
          g_module_close(module);
          g_free(format_module);
        }

      } else {
        libgnac_warning("Impossible to find format module init function for"
          " module %s\n", (gchar *) temp->data);
      }
    }
    temp = temp->next;
  }

  g_list_foreach(list, (GFunc) g_free ,NULL);
  g_list_free(list);
  
}

static GList *
gnac_profiles_properties_find_modules(void)
{
  GList *list = NULL;
  GFile *dir;
  GFileEnumerator *files;
  GFileInfo *file_info;
  GError *error = NULL;
  
  dir = g_file_new_for_path(PLUGINDIR);
  if (!g_file_query_exists(dir,NULL)) {
    libgnac_warning(_("The format modules directory doesn't exits\n"));
    g_error_free(error);
  }
  
  files = g_file_enumerate_children(dir,
      G_FILE_ATTRIBUTE_STANDARD_NAME ","
      G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
      G_FILE_ATTRIBUTE_STANDARD_TYPE,
      G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, &error);

  if (files != NULL) {
    while ((file_info = g_file_enumerator_next_file(files, NULL, NULL)) != NULL) {
      guint32 type;
      const gchar *content_type;

      type = g_file_info_get_file_type(file_info);
      content_type = g_file_info_get_content_type(file_info);

      if ((type == G_FILE_TYPE_REGULAR) 
          && (!g_strcmp0("application/x-sharedlib",content_type))) {
        GFile *module;
        gchar *module_path;
        const gchar *module_name;
        gchar *module_full_path;

        module = g_file_enumerator_get_container(files);
        module_path = g_file_get_path(module);
        module_name = g_file_info_get_name(file_info);
        module_full_path = g_module_build_path(module_path, module_name);
        g_free(module_path);
        list = g_list_append(list, module_full_path);
      }
      g_object_unref(file_info);
    }
  } else {
    libgnac_warning("%s:%s\n", _("Unable to browse to format directory\n"),
      error->message);
    g_error_free(error);
  }
  g_object_unref(dir);
  g_file_enumerator_close(files, NULL, NULL);
  g_object_unref(files);
  
  return list;
}
*/


void
gnac_profiles_properties_show(void *profile, const gchar *title, GCallback call_back)
{
  AudioProfileGeneric *p;
  GtkWidget *widget;
  GtkWidget *format_combo_box;
  FormatModule *format_module;
  GtkTreeIter iter;
  GtkTreePath *tree_path;
  GtkTreeModel *model;
  
  gnac_profiles_properties_reinit();

  widget = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,"save-button"));
  if (g_signal_handler_is_connected(G_OBJECT(widget), save_button_handler_id)) {
    g_signal_handler_disconnect(G_OBJECT(widget), save_button_handler_id);
  }
  save_button_handler_id = g_signal_connect (G_OBJECT (widget), "clicked",
    G_CALLBACK(gnac_profiles_properties_on_save), call_back);
  
  widget = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,"window1"));
  gtk_window_set_title(GTK_WINDOW(widget),title);
  format_combo_box = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,"format_combo_box"));
  if (profile == NULL) {
    gtk_widget_show(widget);
    gtk_combo_box_set_active(GTK_COMBO_BOX(format_combo_box), 0);
  } else {
    p = ((AudioProfileGeneric*)profile)->generic;
    format_module = g_hash_table_lookup(formats,p->format_id);
    if (format_module == NULL) {
      libgnac_warning(_("Format not supported\n"));
    } else {
      model = gtk_combo_box_get_model(GTK_COMBO_BOX(format_combo_box));
      gnac_profiles_properties_name_description_set_text(p->name, p->description);
      format_module->funcs.fullfill_fields(profile);
      tree_path = gtk_tree_row_reference_get_path(format_module->tree_ref); 
      gtk_tree_model_get_iter(model, &iter, tree_path);
      gtk_combo_box_set_active_iter(GTK_COMBO_BOX(format_combo_box), &iter);
      format_module->funcs.generate_pipeline(NULL);
      gtk_widget_show(widget);
      gtk_tree_path_free(tree_path);
    }
  }
}


void
gnac_profiles_properties_free_audio_profile(void *profile)
{
  AudioProfileGeneric *p;
  FormatModule *format_module;
  
  p = ((AudioProfileGeneric*)profile)->generic;
  format_module = g_hash_table_lookup(formats,p->format_id);
  if (format_module == NULL) {
    libgnac_warning(_("Format not supported\n"));
  } else {
    format_module->funcs.free_audio_profile(profile); 
  }
}


void
gnac_profiles_properties_destroy(void)
{
  if (formats) {
    GList *list;
    GList *temp;
    GtkWidget *window;
    FormatModule *format_module;
    
    list = g_hash_table_get_values(formats);
    for (temp = list; temp != NULL; temp = temp->next) {
      format_module = ((FormatModule*)temp->data);
      format_module->funcs.clean_up();
      //g_module_close(format_module->module);
      g_free(format_module);
    }
    g_list_free(list);
    g_hash_table_destroy(formats);
    formats = NULL;
    g_free(saved_profiles_dir);
    window = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,"window1"));
    g_object_unref(G_OBJECT(profiles_properties_builder));
    gtk_widget_destroy(window);
  }
}

static void
gnac_profiles_properties_combo_format_on_changed(GtkComboBox *widget, gpointer user_data)
{
  GtkWidget *properties;
  GtkWidget *hbox_properties;
  GtkWidget *label;
  GtkWidget *text_view;
  GtkTreeModel *model;
  GtkTreeIter iter;
  FormatModule *format_module;
  
  model = gtk_combo_box_get_model(widget);
  if (gtk_combo_box_get_active_iter(widget, &iter)) {
    gtk_tree_model_get(model,&iter,1,(void**)&format_module,-1);

    gnac_profiles_properties_display_status_message(NULL);
    hbox_properties = GTK_WIDGET(user_data);
    
    if  ((current_format_module != NULL)
        && (format_module != current_format_module)) {
      properties = current_format_module->funcs.get_widget(NULL);
      gtk_widget_hide(properties);
    }

    text_view = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
      "gstreamer_pipeline_text_view"));
    gtk_text_view_set_editable(GTK_TEXT_VIEW(text_view), FALSE);
    #ifndef GNOME_ENABLE_DEBUG 
    text_view = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
      "gstreamer_pipeline_box"));
    gtk_widget_hide(text_view);
    #endif /* GNOME_ENABLE_DEBUG */

    label = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
    "format_decription_label"));
    gtk_label_set_markup(GTK_LABEL(label), format_module->funcs.get_description());
    properties = format_module->funcs.get_widget(NULL);
    gtk_widget_show(properties);
    format_module->funcs.generate_pipeline(NULL);
    current_format_module = format_module;
  }
}


static void
gnac_profiles_properties_update_textbuffer(const gchar *pipeline)
{
  GtkWidget *text_view;
  GtkTextBuffer *pipeline_text_buffer;

  text_view = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
    "gstreamer_pipeline_text_view"));
  pipeline_text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view));
  gtk_text_buffer_set_text(pipeline_text_buffer,pipeline,strlen(pipeline));
}

static void
gnac_profiles_properties_display_status_message(const gchar *error)
{
  GtkWidget *widget;
  
  widget = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
    "hbox-properties-status"));
  if (error != NULL) {
    gtk_widget_show_all(widget);

    widget = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
    "label-properties-status-error"));
    gtk_label_set_text(GTK_LABEL(widget), error);
 
  } else {
    gtk_widget_hide_all(widget); 
  }
}

static void
gnac_profiles_properties_reinit() {
  GList *list;
  GList *temp;

  gnac_profiles_properties_update_textbuffer("");
  gnac_profiles_properties_name_description_set_text("","");
  list = g_hash_table_get_values(formats);
  for (temp = list; temp != NULL; temp = temp->next) {
    ((FormatModule*)temp->data)->funcs.fullfill_fields(NULL);  
  }
  g_list_free(list);
}

static void
gnac_profiles_properties_name_description_set_text(const gchar *name,
  const gchar *description)
{ 
  GtkWidget *widget;
  GtkTextBuffer *text_buffer;
  
  if (name != NULL) {
    widget = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
        "name_entry"));
    gtk_entry_set_text(GTK_ENTRY(widget),name);
    current_profile_name = name;
  }
  
  if (description != NULL) {
    widget = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
      "profile_description_text_view"));
    text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
    gtk_text_buffer_set_text(GTK_TEXT_BUFFER(text_buffer),description,
      strlen(description));
  }
}

static void
gnac_profiles_properties_on_save(GtkWidget *widget, gpointer data)
{
  GtkWidget *window;
  GtkTextBuffer *text_buffer;
  GtkTextIter start;
  GtkTextIter end;
  const gchar *name;
  StandardCallBack call_back = (StandardCallBack) data;
  void *profile;
  AudioProfileGeneric *generic;
  GError *error = NULL;

  window = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,"window1"));
  
  profile = current_format_module->funcs.generate_audio_profile(&error);
  if (profile == NULL) {
    if (error) {
      gnac_profiles_properties_display_status_message(error->message);
      g_error_free(error);
    }
  } else {
    GtkWidget *text_view;
    GtkTextBuffer *pipeline_text_buffer;

    text_view = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
      "gstreamer_pipeline_text_view"));
    pipeline_text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view));

    generic = ((AudioProfileGeneric*) profile)->generic;
    
    if (generic->pipeline == NULL) {
      gtk_text_buffer_get_start_iter(pipeline_text_buffer,&start);
      gtk_text_buffer_get_end_iter(pipeline_text_buffer,&end);
      generic->pipeline = gtk_text_buffer_get_text(pipeline_text_buffer,
        &start,&end,FALSE);
    }
    
    widget = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
        "name_entry"));
    name = gtk_entry_get_text(GTK_ENTRY(widget));
    
    if (!g_strcmp0(name,"")) {
      gnac_profiles_properties_display_status_message(_("The profile name must be non"
        "-empty"));
    } else if (!gnac_profiles_properties_is_valid_filename_chars(name)) {
      gchar *message;

      message = g_strconcat(_("Profile name cannot contain the following "
        "character(s):"), forbidden_chars_string, NULL);
      gnac_profiles_properties_display_status_message(message);
      g_free(message);
    } else if (g_strcmp0(current_profile_name, name)
      && (gnac_profiles_properties_saved_profiles_contain_name(name,TRUE))) {
        
        gnac_profiles_properties_display_status_message(_("This name is already used "
          "by another profile."));

    } else {
      gnac_profiles_properties_display_status_message(NULL);
      
      generic->name = g_strdup(name);
      widget = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
        "profile_description_text_view"));
      text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
      gtk_text_buffer_get_start_iter(text_buffer,&start);
      gtk_text_buffer_get_end_iter(text_buffer,&end);
      generic->description = gtk_text_buffer_get_text(text_buffer,&start,&end,FALSE);

      current_format_module->funcs.save_profile_in_file(profile);
      
      call_back(widget, profile);

      gtk_widget_hide(window);
    }
  }
}

gboolean
gnac_profiles_properties_on_name_entry_key_pressed(GtkWidget *widget,
  GdkEventKey *event, gpointer data)
{
  if (event->keyval == GDK_Return) {
    GtkWidget *widget;

    widget = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
      "save-button"));
    gtk_button_clicked(GTK_BUTTON(widget));
    return TRUE;
  } else if (gnac_profiles_properties_is_valid_filename_char(event->keyval)) {
    gnac_profiles_properties_display_status_message(NULL);
    return FALSE;
  } else {
    gchar *message;
    
    message = g_strconcat(_("Profile name cannot contain the following "
      "character(s):"), forbidden_chars_string, NULL);
    gnac_profiles_properties_display_status_message(message);
    g_free(message);
    return TRUE;
  }
}

gboolean
gnac_profiles_properties_on_window_key_pressed(GtkWidget *widget,
  GdkEventKey *event, gpointer data)
{
  if (event->keyval == GDK_Escape) {
    GtkWidget *widget;

    widget = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
      "cancel-button"));
    gtk_button_clicked(GTK_BUTTON(widget));
    return TRUE;
  } else {
    return FALSE;
  }
}

static gboolean
gnac_profiles_properties_is_valid_filename_char(guint keyval)
{
  guint i;

  for (i = 0; i<nb_forbidden_chars; ++i) {
    if (keyval == forbidden_chars_keys[i]) {
      return FALSE;
    }
  }

  return TRUE;
}

static gboolean
gnac_profiles_properties_is_valid_filename_chars(const gchar *chars) {
  size_t len;
  int i;
  int j;
  
  len = strlen(chars);
  for (i = 0; i<len; ++i) {
    for (j = 0; j<nb_forbidden_chars; ++j) {
      if (chars[i] == forbidden_chars[j]) {
        return FALSE;
      }
    }
  }
  return TRUE;
}


void
gnac_profiles_properties_on_cancel(GtkWidget *widget, gpointer data)
{
  GtkWidget *window;

  window = GTK_WIDGET(gtk_builder_get_object(profiles_properties_builder,
    "window1"));
  gtk_widget_hide(window);
}

void *
gnac_profiles_properties_load_profile_from_file(const gchar *path,
  const gchar *filename, GError **error)
{
  XMLDoc *doc;
  AudioProfileGeneric *profile;
  FormatModule *format_module;

  doc = gnac_profiles_default_load_generic_audio_profile(path, &profile);
  if ((doc == NULL)  || (profile->format_id == NULL)) {
    if ((error) && (*error == NULL)) { 
      *error = g_error_new(g_quark_from_static_string("gnac")
        ,0,_("File \"%s\" is not a valid profile file."),
      filename); 
    }
  } else {

    format_module = g_hash_table_lookup(formats,profile->format_id);
    if (format_module == NULL) {
      if ((error) && (*error == NULL)) { 
        *error = g_error_new(g_quark_from_static_string("gnac"),
          0,_("Format defined by id \"%s\" in file "
          "\"%s\" is not supported."), profile->format_id, filename);
      }
      gnac_profiles_utils_free_audio_profile_generic(profile);
      profile = NULL;
    } else {
      profile = format_module->funcs.load_specific_properties(doc,profile);
    }
  }
  
  if (doc != NULL) gnac_profiles_xml_engine_free_doc_xpath(doc);
  return profile;
}

gboolean
gnac_profiles_properties_saved_profiles_contain_name(const gchar *name,
  gboolean add_ext)
{
  gchar *path;
  GFile *file;
  gboolean exists;
  
  if (!add_ext) {
    path = g_strconcat(GNAC_SAVED_PROFILES_URL(name), NULL);
  } else {
    path = g_strconcat(GNAC_SAVED_PROFILES_URL_WITH_EXT(name), NULL);
  }
  file = g_file_new_for_path(path);
  exists = g_file_query_exists(file,NULL);

  g_free(path);
  g_object_unref(file);
  return exists; 
}

void
gnac_profiles_properties_save_profile(void *profile) {
  const gchar *format_id;
  FormatModule *format_module;

  format_id = ((AudioProfileGeneric*)profile)->generic->format_id;
  format_module = g_hash_table_lookup(formats, format_id);
  if (format_module != NULL) {
    format_module->funcs.save_profile_in_file(profile);
  }
}

gchar *
gnac_profiles_properties_filter_text_for_displaying(const gchar *text, int length) {
  gchar *str1;
  gchar *str2;
  size_t len;
  
  if (text != NULL) {
    len = strlen(text);
    if (len > length) {
      str2 = g_strndup(text, length);
      str1 = g_strconcat(str2,"...", NULL);
      g_free(str2);
    } else {
      str1 = g_strdup(text);
    }
    g_strdelimit(str1,"\n\t\r\b\f",' ');
    return str1;
  } else {
    return NULL;
  }
}
